@@ -18,6 +18,7 @@
 package org.apache.spark.shuffle.celeborn;
 
 import java.io.IOException;
+import java.util.concurrent.LinkedBlockingQueue;
 import java.util.concurrent.atomic.LongAdder;
 
 import javax.annotation.Nullable;
@@ -39,11 +40,10 @@ import org.apache.spark.serializer.SerializerInstance;
 import org.apache.spark.shuffle.ShuffleWriteMetricsReporter;
 import org.apache.spark.shuffle.ShuffleWriter;
 import org.apache.spark.sql.catalyst.expressions.UnsafeRow;
-import org.apache.spark.sql.execution.PartitionIdPassthrough;
 import org.apache.spark.sql.execution.UnsafeRowSerializer;
-import org.apache.spark.sql.execution.columnar.RssBatchBuilder;
-import org.apache.spark.sql.execution.columnar.RssColumnarBatchBuilder;
-import org.apache.spark.sql.execution.columnar.RssColumnarBatchCodeGenBuild;
+import org.apache.spark.sql.execution.columnar.CelebornBatchBuilder;
+import org.apache.spark.sql.execution.columnar.CelebornColumnarBatchBuilder;
+import org.apache.spark.sql.execution.columnar.CelebornColumnarBatchCodeGenBuild;
 import org.apache.spark.sql.execution.metric.SQLMetric;
 import org.apache.spark.sql.types.StructType;
 import org.apache.spark.storage.BlockManagerId;
@@ -53,7 +53,9 @@ import org.slf4j.LoggerFactory;
 
 import org.apache.celeborn.client.ShuffleClient;
 import org.apache.celeborn.client.write.DataPusher;
+import org.apache.celeborn.client.write.PushTask;
 import org.apache.celeborn.common.CelebornConf;
+import org.apache.celeborn.common.util.Utils;
 
 @Private
 public class HashBasedShuffleWriter<K, V, C> extends ShuffleWriter<K, V> {
@@ -68,11 +70,10 @@ public class HashBasedShuffleWriter<K, V, C> extends ShuffleWriter<K, V> {
   private final ShuffleDependency<K, V, C> dep;
   private final Partitioner partitioner;
   private final ShuffleWriteMetricsReporter writeMetrics;
-  private final String appId;
   private final int shuffleId;
   private final int mapId;
   private final TaskContext taskContext;
-  private final ShuffleClient rssShuffleClient;
+  private final ShuffleClient shuffleClient;
   private final int numMappers;
   private final int numPartitions;
 
@@ -89,7 +90,7 @@ public class HashBasedShuffleWriter<K, V, C> extends ShuffleWriter<K, V> {
   private final LongAdder[] mapStatusLengths;
   private final long[] tmpRecords;
 
-  private RssBatchBuilder[] rssBatchBuilders;
+  private CelebornBatchBuilder[] celebornBatchBuilders;
   private final SendBufferPool sendBufferPool;
 
   /**
@@ -99,16 +100,30 @@ public class HashBasedShuffleWriter<K, V, C> extends ShuffleWriter<K, V> {
    */
   private volatile boolean stopping = false;
 
-  private final DataPusher dataPusher;
+  private DataPusher dataPusher;
 
   private StructType schema;
 
+  private final boolean unsafeRowFastWrite;
+
+  // //////////////////////////////////////////////////////
+  //                 Columnar Relate Conf                //
+  // //////////////////////////////////////////////////////
+
   private boolean isColumnarShuffle = false;
 
+  private int columnarShuffleBatchSize;
+
+  private boolean columnarShuffleCodeGenEnabled;
+
+  private boolean columnarShuffleDictionaryEnabled;
+
+  private double columnarShuffleDictionaryMaxFactor;
+
   // In order to facilitate the writing of unit test code, ShuffleClient needs to be passed in as
   // parameters. By the way, simplify the passed parameters.
   public HashBasedShuffleWriter(
-      RssShuffleHandle<K, V, C> handle,
+      CelebornShuffleHandle<K, V, C> handle,
       TaskContext taskContext,
       CelebornConf conf,
       ShuffleClient client,
@@ -117,7 +132,6 @@ public class HashBasedShuffleWriter<K, V, C> extends ShuffleWriter<K, V> {
       throws IOException {
     this.mapId = taskContext.partitionId();
     this.dep = handle.dependency();
-    this.appId = handle.newAppId();
     this.shuffleId = dep.shuffleId();
     SerializerInstance serializer = dep.serializer().newInstance();
     this.partitioner = dep.partitioner();
@@ -125,9 +139,10 @@ public class HashBasedShuffleWriter<K, V, C> extends ShuffleWriter<K, V> {
     this.taskContext = taskContext;
     this.numMappers = handle.numMappers();
     this.numPartitions = dep.partitioner().numPartitions();
-    this.rssShuffleClient = client;
+    this.shuffleClient = client;
     this.conf = conf;
 
+    unsafeRowFastWrite = conf.clientPushUnsafeRowFastWrite();
     serBuffer = new OpenByteArrayOutputStream(DEFAULT_INITIAL_SER_BUFFER_SIZE);
     serOutputStream = serializer.serializeStream(serBuffer);
 
@@ -137,62 +152,76 @@ public class HashBasedShuffleWriter<K, V, C> extends ShuffleWriter<K, V> {
     }
     tmpRecords = new long[numPartitions];
 
-    PUSH_BUFFER_INIT_SIZE = conf.pushBufferInitialSize();
-    PUSH_BUFFER_MAX_SIZE = conf.pushBufferMaxSize();
+    PUSH_BUFFER_INIT_SIZE = conf.clientPushBufferInitialSize();
+    PUSH_BUFFER_MAX_SIZE = conf.clientPushBufferMaxSize();
 
     this.sendBufferPool = sendBufferPool;
     sendBuffers = sendBufferPool.acquireBuffer(numPartitions);
-    if (sendBuffers == null) {
-      logger.info("Acquire failed");
-      sendBuffers = new byte[numPartitions][];
-    }
     sendOffsets = new int[numPartitions];
 
-    dataPusher =
-        new DataPusher(
-            appId,
-            shuffleId,
-            mapId,
-            taskContext.attemptNumber(),
-            taskContext.taskAttemptId(),
-            numMappers,
-            numPartitions,
-            conf,
-            rssShuffleClient,
-            writeMetrics::incBytesWritten,
-            mapStatusLengths);
+    try {
+      LinkedBlockingQueue<PushTask> pushTaskQueue = sendBufferPool.acquirePushTaskQueue();
+      dataPusher =
+          new DataPusher(
+              shuffleId,
+              mapId,
+              taskContext.attemptNumber(),
+              taskContext.taskAttemptId(),
+              numMappers,
+              numPartitions,
+              conf,
+              shuffleClient,
+              pushTaskQueue,
+              writeMetrics::incBytesWritten,
+              mapStatusLengths);
+    } catch (InterruptedException e) {
+      TaskInterruptedHelper.throwTaskKillException();
+    }
 
     if (conf.columnarShuffleEnabled()) {
+      columnarShuffleBatchSize = conf.columnarShuffleBatchSize();
+      columnarShuffleCodeGenEnabled = conf.columnarShuffleCodeGenEnabled();
+      columnarShuffleDictionaryEnabled = conf.columnarShuffleDictionaryEnabled();
+      columnarShuffleDictionaryMaxFactor = conf.columnarShuffleDictionaryMaxFactor();
       this.schema = SparkUtils.getSchema(dep);
-      this.rssBatchBuilders = new RssBatchBuilder[numPartitions];
-      this.isColumnarShuffle = RssBatchBuilder.supportsColumnarType(schema);
+      this.celebornBatchBuilders = new CelebornBatchBuilder[numPartitions];
+      this.isColumnarShuffle = CelebornBatchBuilder.supportsColumnarType(schema);
     }
   }
 
   @Override
   public void write(scala.collection.Iterator<Product2<K, V>> records) throws IOException {
-    if (canUseFastWrite()) {
-      if (isColumnarShuffle) {
-        fastColumnarWrite0(records);
+    try {
+      if (canUseFastWrite()) {
+        if (isColumnarShuffle) {
+          fastColumnarWrite0(records);
+        } else {
+          fastWrite0(records);
+        }
+      } else if (dep.mapSideCombine()) {
+        if (dep.aggregator().isEmpty()) {
+          throw new UnsupportedOperationException(
+              "When using map side combine, an aggregator must be specified.");
+        }
+        write0(dep.aggregator().get().combineValuesByKey(records, taskContext));
       } else {
-        fastWrite0(records);
+        write0(records);
       }
-    } else if (dep.mapSideCombine()) {
-      if (dep.aggregator().isEmpty()) {
-        throw new UnsupportedOperationException(
-            "When using map side combine, an aggregator must be specified.");
-      }
-      write0(dep.aggregator().get().combineValuesByKey(records, taskContext));
-    } else {
-      write0(records);
+      close();
+    } catch (InterruptedException e) {
+      TaskInterruptedHelper.throwTaskKillException();
     }
-    close();
   }
 
   @VisibleForTesting
   boolean canUseFastWrite() {
-    return dep.serializer() instanceof UnsafeRowSerializer
-        && partitioner instanceof PartitionIdPassthrough;
+    boolean keyIsPartitionId = false;
+    if (unsafeRowFastWrite && dep.serializer() instanceof UnsafeRowSerializer) {
+      // SPARK-39391 renames PartitionIdPassthrough's package
+      String partitionerClassName = partitioner.getClass().getSimpleName();
+      keyIsPartitionId = "PartitionIdPassthrough".equals(partitionerClassName);
+    }
+    return keyIsPartitionId;
   }
 
   private void fastColumnarWrite0(scala.collection.Iterator iterator) throws IOException {
@@ -204,36 +233,38 @@ public class HashBasedShuffleWriter<K, V, C> extends ShuffleWriter<K, V> {
       final int partitionId = record._1();
       final UnsafeRow row = record._2();
 
-      if (rssBatchBuilders[partitionId] == null) {
-        RssBatchBuilder columnBuilders;
-        if (conf.columnarShuffleCodeGenEnabled() && !conf.columnarShuffleDictionaryEnabled()) {
+      if (celebornBatchBuilders[partitionId] == null) {
+        CelebornBatchBuilder columnBuilders;
+        if (columnarShuffleCodeGenEnabled && !columnarShuffleDictionaryEnabled) {
           columnBuilders =
-              new RssColumnarBatchCodeGenBuild().create(schema, conf.columnarShuffleBatchSize());
+              new CelebornColumnarBatchCodeGenBuild().create(schema, columnarShuffleBatchSize);
         } else {
           columnBuilders =
-              new RssColumnarBatchBuilder(
+              new CelebornColumnarBatchBuilder(
                   schema,
-                  conf.columnarShuffleBatchSize(),
-                  conf.columnarShuffleDictionaryMaxFactor(),
-                  conf.columnarShuffleDictionaryEnabled());
+                  columnarShuffleBatchSize,
+                  columnarShuffleDictionaryMaxFactor,
+                  columnarShuffleDictionaryEnabled);
         }
         columnBuilders.newBuilders();
-        rssBatchBuilders[partitionId] = columnBuilders;
+        celebornBatchBuilders[partitionId] = columnBuilders;
       }
-      rssBatchBuilders[partitionId].writeRow(row);
-      if (rssBatchBuilders[partitionId].getRowCnt() >= conf.columnarShuffleBatchSize()) {
-        byte[] arr = rssBatchBuilders[partitionId].buildColumnBytes();
+
+      celebornBatchBuilders[partitionId].writeRow(row);
+      if (celebornBatchBuilders[partitionId].getRowCnt() >= columnarShuffleBatchSize) {
+        byte[] arr = celebornBatchBuilders[partitionId].buildColumnBytes();
         pushGiantRecord(partitionId, arr, arr.length);
         if (dataSize != null) {
           dataSize.add(arr.length);
         }
-        rssBatchBuilders[partitionId].newBuilders();
+        celebornBatchBuilders[partitionId].newBuilders();
       }
       tmpRecords[partitionId] += 1;
     }
   }
 
-  private void fastWrite0(scala.collection.Iterator iterator) throws IOException {
+  private void fastWrite0(scala.collection.Iterator iterator)
+      throws IOException, InterruptedException {
     final scala.collection.Iterator<Product2<Integer, UnsafeRow>> records = iterator;
 
     SQLMetric dataSize = SparkUtils.getDataSize((UnsafeRowSerializer) dep.serializer());
@@ -275,7 +306,7 @@ public class HashBasedShuffleWriter<K, V, C> extends ShuffleWriter<K, V> {
     }
   }
 
-  private void write0(scala.collection.Iterator iterator) throws IOException {
+  private void write0(scala.collection.Iterator iterator) throws IOException, InterruptedException {
     final scala.collection.Iterator<Product2<K, ?>> records = iterator;
 
     while (records.hasNext()) {
@@ -314,10 +345,8 @@ public class HashBasedShuffleWriter<K, V, C> extends ShuffleWriter<K, V> {
 
   private void pushGiantRecord(int partitionId, byte[] buffer, int numBytes) throws IOException {
     logger.debug("Push giant record, size {}.", numBytes);
-    long pushStartTime = System.nanoTime();
     int bytesWritten =
-        rssShuffleClient.pushData(
-            appId,
+        shuffleClient.pushData(
             shuffleId,
             mapId,
             taskContext.attemptNumber(),
@@ -329,10 +358,10 @@ public class HashBasedShuffleWriter<K, V, C> extends ShuffleWriter<K, V> {
             numPartitions);
     mapStatusLengths[partitionId].add(bytesWritten);
     writeMetrics.incBytesWritten(bytesWritten);
-    writeMetrics.incWriteTime(System.nanoTime() - pushStartTime);
   }
 
-  private int getOrUpdateOffset(int partitionId, int serializedRecordSize) throws IOException {
+  private int getOrUpdateOffset(int partitionId, int serializedRecordSize)
+      throws IOException, InterruptedException {
     int offset = sendOffsets[partitionId];
     byte[] buffer = getOrCreateBuffer(partitionId);
     while ((buffer.length - offset) < serializedRecordSize
@@ -353,25 +382,25 @@ public class HashBasedShuffleWriter<K, V, C> extends ShuffleWriter<K, V> {
     return offset;
   }
 
-  private void flushSendBuffer(int partitionId, byte[] buffer, int size) throws IOException {
-    long pushStartTime = System.nanoTime();
-    logger.debug("Flush buffer, size {}.", size);
+  private void flushSendBuffer(int partitionId, byte[] buffer, int size)
+      throws IOException, InterruptedException {
+    long start = System.nanoTime();
+    logger.debug("Flush buffer, size {}.", Utils.bytesToString(size));
     dataPusher.addTask(partitionId, buffer, size);
-    writeMetrics.incWriteTime(System.nanoTime() - pushStartTime);
+    writeMetrics.incWriteTime(System.nanoTime() - start);
   }
 
   private void closeColumnarWrite() throws IOException {
     SQLMetric dataSize = SparkUtils.getDataSize((UnsafeRowSerializer) dep.serializer());
     for (int i = 0; i < numPartitions; i++) {
-      final RssBatchBuilder buidlers = rssBatchBuilders[i];
+      final CelebornBatchBuilder buidlers = celebornBatchBuilders[i];
       if (buidlers != null && buidlers.getRowCnt() > 0) {
         byte[] buffers = buidlers.buildColumnBytes();
         if (dataSize != null) {
           dataSize.add(buffers.length);
         }
         int bytesWritten =
-            rssShuffleClient.mergeData(
-                appId,
+            shuffleClient.mergeData(
                 shuffleId,
                 mapId,
                 taskContext.attemptNumber(),
@@ -382,7 +411,7 @@ public class HashBasedShuffleWriter<K, V, C> extends ShuffleWriter<K, V> {
                 numMappers,
                 numPartitions);
         // free buffer
-        rssBatchBuilders[i] = null;
+        celebornBatchBuilders[i] = null;
         mapStatusLengths[i].add(bytesWritten);
         writeMetrics.incBytesWritten(bytesWritten);
       }
@@ -397,8 +426,7 @@ public class HashBasedShuffleWriter<K, V, C> extends ShuffleWriter<K, V> {
       final int size = sendOffsets[i];
       if (size > 0) {
         int bytesWritten =
-            rssShuffleClient.mergeData(
-                appId,
+            shuffleClient.mergeData(
                 shuffleId,
                 mapId,
                 taskContext.attemptNumber(),
@@ -419,23 +447,24 @@ public class HashBasedShuffleWriter<K, V, C> extends ShuffleWriter<K, V> {
     sendOffsets = null;
   }
 
-  private void close() throws IOException {
+  private void close() throws IOException, InterruptedException {
     // here we wait for all the in-flight batches to return which sent by dataPusher thread
     long pushMergedDataTime = System.nanoTime();
     dataPusher.waitOnTermination();
-    rssShuffleClient.prepareForMergeData(shuffleId, mapId, taskContext.attemptNumber());
+    sendBufferPool.returnPushTaskQueue(dataPusher.getIdleQueue());
+    shuffleClient.prepareForMergeData(shuffleId, mapId, taskContext.attemptNumber());
     if (isColumnarShuffle) {
       closeColumnarWrite();
     } else {
       closeRowWrite();
     }
-    rssShuffleClient.pushMergedData(appId, shuffleId, mapId, taskContext.attemptNumber());
+    shuffleClient.pushMergedData(shuffleId, mapId, taskContext.attemptNumber());
     writeMetrics.incWriteTime(System.nanoTime() - pushMergedDataTime);
 
     updateMapStatus();
 
     long waitStartTime = System.nanoTime();
-    rssShuffleClient.mapperEnd(appId, shuffleId, mapId, taskContext.attemptNumber(), numMappers);
+    shuffleClient.mapperEnd(shuffleId, mapId, taskContext.attemptNumber(), numMappers);
     writeMetrics.incWriteTime(System.nanoTime() - waitStartTime);
 
     BlockManagerId bmId = SparkEnv.get().blockManager().shuffleServerId();
@@ -472,12 +501,12 @@ public class HashBasedShuffleWriter<K, V, C> extends ShuffleWriter<K, V> {
         }
       }
     } finally {
-      rssShuffleClient.cleanup(appId, shuffleId, mapId, taskContext.attemptNumber());
+      shuffleClient.cleanup(shuffleId, mapId, taskContext.attemptNumber());
     }
   }
 
   public long[] getPartitionLengths() {
     throw new UnsupportedOperationException(
-        "RSS is not compatible with Spark push mode, please set spark.shuffle.push.enabled to false");
+        "Celeborn is not compatible with Spark push mode, please set spark.shuffle.push.enabled to false");
   }
 }
