@@ -17,43 +17,42 @@
 
 package org.apache.celeborn.common.util
 
-import java.io.{File, FileInputStream, InputStreamReader, IOException}
+import java.io.{File, FileInputStream, InputStream, InputStreamReader, IOException}
 import java.lang.management.ManagementFactory
 import java.math.{MathContext, RoundingMode}
 import java.net._
 import java.nio.ByteBuffer
+import java.nio.channels.FileChannel
 import java.nio.charset.StandardCharsets
-import java.text.SimpleDateFormat
 import java.util
 import java.util.{Locale, Properties, Random, UUID}
-import java.util.concurrent.{Callable, ConcurrentHashMap, ThreadPoolExecutor, TimeoutException, TimeUnit}
+import java.util.concurrent.{Callable, ThreadPoolExecutor, TimeoutException, TimeUnit}
 
+import scala.annotation.tailrec
 import scala.collection.JavaConverters._
+import scala.io.Source
 import scala.reflect.ClassTag
 import scala.util.Try
 import scala.util.control.{ControlThrowable, NonFatal}
 
-import com.google.common.net.InetAddresses
 import com.google.protobuf.{ByteString, GeneratedMessageV3}
 import io.netty.channel.unix.Errors.NativeIoException
 import org.apache.commons.lang3.SystemUtils
+import org.apache.hadoop.fs.Path
 import org.roaringbitmap.RoaringBitmap
 
 import org.apache.celeborn.common.CelebornConf
 import org.apache.celeborn.common.exception.CelebornException
 import org.apache.celeborn.common.internal.Logging
-import org.apache.celeborn.common.meta.{DiskStatus, WorkerInfo}
+import org.apache.celeborn.common.meta.{DiskStatus, FileInfo, WorkerInfo}
 import org.apache.celeborn.common.network.protocol.TransportMessage
-import org.apache.celeborn.common.network.util.{ConfigProvider, JavaUtils, TransportConf}
-import org.apache.celeborn.common.protocol.{PartitionLocation, PartitionSplitMode, PartitionType}
-import org.apache.celeborn.common.protocol.PbWorkerResource
+import org.apache.celeborn.common.network.util.TransportConf
+import org.apache.celeborn.common.protocol.{PartitionLocation, PartitionSplitMode, PartitionType, TransportModuleConstants}
 import org.apache.celeborn.common.protocol.message.{ControlMessages, Message, StatusCode}
 import org.apache.celeborn.common.protocol.message.ControlMessages.WorkerResource
 
 object Utils extends Logging {
 
-  def createDateFormat: SimpleDateFormat = new SimpleDateFormat("yyyyMMddHHmmss", Locale.US)
-
   def stringToSeq(str: String): Seq[String] = {
     str.split(",").map(_.trim()).filter(_.nonEmpty)
   }
@@ -93,38 +92,31 @@ object Utils extends Logging {
     (JavaUtils.byteStringAsBytes(str) / 1024 / 1024).toInt
   }
 
-  def bytesToString(size: Long): String = bytesToString(BigInt(size))
+  private[this] val siByteSizes =
+    Array(1L << 60, 1L << 50, 1L << 40, 1L << 30, 1L << 20, 1L << 10, 1)
+  private[this] val siByteSuffixes =
+    Array("EiB", "PiB", "TiB", "GiB", "MiB", "KiB", "B")
+
+  /**
+   * Convert a quantity in bytes to a human-readable string such as "4.0 MiB".
+   */
+  def bytesToString(size: Long): String = {
+    var i = 0
+    while (i < siByteSizes.length - 1 && size < 2 * siByteSizes(i)) i += 1
+    "%.1f %s".formatLocal(Locale.US, size.toDouble / siByteSizes(i), siByteSuffixes(i))
+  }
 
   def bytesToString(size: BigInt): String = {
-    val EB = 1L << 60
-    val PB = 1L << 50
-    val TB = 1L << 40
-    val GB = 1L << 30
-    val MB = 1L << 20
-    val KB = 1L << 10
-
-    if (size >= BigInt(1L << 11) * EB) {
+    val EiB = 1L << 60
+    if (size.isValidLong) {
+      // Common case, most sizes fit in 64 bits and all ops on BigInt are order(s) of magnitude
+      // slower than Long/Double.
+      bytesToString(size.toLong)
+    } else if (size < BigInt(2L << 10) * EiB) {
+      "%.1f EiB".formatLocal(Locale.US, BigDecimal(size) / EiB)
+    } else {
       // The number is too large, show it in scientific notation.
       BigDecimal(size, new MathContext(3, RoundingMode.HALF_UP)).toString() + " B"
-    } else {
-      val (value, unit) = {
-        if (size >= 2 * EB) {
-          (BigDecimal(size) / EB, "EB")
-        } else if (size >= 2 * PB) {
-          (BigDecimal(size) / PB, "PB")
-        } else if (size >= 2 * TB) {
-          (BigDecimal(size) / TB, "TB")
-        } else if (size >= 2 * GB) {
-          (BigDecimal(size) / GB, "GB")
-        } else if (size >= 2 * MB) {
-          (BigDecimal(size) / MB, "MB")
-        } else if (size >= 2 * KB) {
-          (BigDecimal(size) / KB, "KB")
-        } else {
-          (BigDecimal(size), "B")
-        }
-      }
-      "%.1f %s".formatLocal(Locale.US, value, unit)
     }
   }
 
@@ -153,25 +145,69 @@ object Utils extends Logging {
     }
   }
 
+  def nanoDurationToString(ns: Long): String = {
+    val ms = 1000 * 1000L
+    val second = 1000 * ms
+    val minute = 60 * second
+    val hour = 60 * minute
+    val locale = Locale.US
+
+    ns match {
+      case t if t < ms =>
+        "%d ns".formatLocal(locale, t)
+      case t if t < second =>
+        "%.1f ms".formatLocal(locale, t.toFloat / ms)
+      case t if t < minute =>
+        "%.1f s".formatLocal(locale, t.toFloat / second)
+      case t if t < hour =>
+        "%.1f m".formatLocal(locale, t.toFloat / minute)
+      case t =>
+        "%.2f h".formatLocal(locale, t.toFloat / hour)
+    }
+  }
+
   @throws(classOf[CelebornException])
-  def extractHostPortFromRssUrl(essUrl: String): (String, Int) = {
+  def extractHostPortFromCelebornUrl(celebornUrl: String): (String, Int) = {
     try {
-      val uri = new java.net.URI(essUrl)
+      val uri = new java.net.URI(celebornUrl)
       val host = uri.getHost
       val port = uri.getPort
-      if (uri.getScheme != "rss" ||
+      if (uri.getScheme != "celeborn" ||
         host == null ||
         port < 0 ||
-        (uri.getPath != null && !uri.getPath.isEmpty) || // uri.getPath returns "" instead of null
+        (uri.getPath != null && uri.getPath.nonEmpty) || // uri.getPath returns "" instead of null
         uri.getFragment != null ||
         uri.getQuery != null ||
         uri.getUserInfo != null) {
-        throw new CelebornException("Invalid master URL: " + essUrl)
+        throw new CelebornException(s"Invalid master URL: $celebornUrl")
       }
       (host, port)
     } catch {
       case e: java.net.URISyntaxException =>
-        throw new CelebornException("Invalid master URL: " + essUrl, e)
+        throw new CelebornException(s"Invalid master URL: $celebornUrl", e)
+    }
+  }
+
+  @throws(classOf[CelebornException])
+  def extractHostPortNameFromCelebornUrl(celebornUrl: String): (String, Int, String) = {
+    try {
+      val uri = new java.net.URI(celebornUrl)
+      val host = uri.getHost
+      val port = uri.getPort
+      val name = uri.getUserInfo
+      if (uri.getScheme != "celeborn" ||
+        host == null ||
+        port < 0 ||
+        name == null ||
+        (uri.getPath != null && uri.getPath.nonEmpty) || // uri.getPath returns "" instead of null
+        uri.getFragment != null ||
+        uri.getQuery != null) {
+        throw new CelebornException(s"Invalid Celeborn URL: $celebornUrl")
+      }
+      (host, port, name)
+    } catch {
+      case e: java.net.URISyntaxException =>
+        throw new CelebornException(s"Invalid Celeborn URL: $celebornUrl", e)
     }
   }
 
@@ -188,7 +224,7 @@ object Utils extends Logging {
     }
   }
 
-  def tryLogNonFatalError(block: => Unit) {
+  def tryLogNonFatalError(block: => Unit): Unit = {
     try {
       block
     } catch {
@@ -197,7 +233,7 @@ object Utils extends Logging {
     }
   }
 
-  def tryOrExit(block: => Unit) {
+  def tryOrExit(block: => Unit): Unit = {
     try {
       block
     } catch {
@@ -206,41 +242,6 @@ object Utils extends Logging {
     }
   }
 
-  def tryWithSafeFinallyAndFailureCallbacks[T](block: => T)(
-      catchBlock: => Unit = (),
-      finallyBlock: => Unit = ()): T = {
-    var originalThrowable: Throwable = null
-    try {
-      block
-    } catch {
-      case cause: Throwable =>
-        // Purposefully not using NonFatal, because even fatal exceptions
-        // we don't want to have our finallyBlock suppress
-        originalThrowable = cause
-        try {
-          logError("Aborting task", originalThrowable)
-          //          TaskContext.get().markTaskFailed(originalThrowable)
-          catchBlock
-        } catch {
-          case t: Throwable =>
-            if (originalThrowable != t) {
-              originalThrowable.addSuppressed(t)
-              logWarning(s"Suppressing exception in catch: ${t.getMessage}", t)
-            }
-        }
-        throw originalThrowable
-    } finally {
-      try {
-        finallyBlock
-      } catch {
-        case t: Throwable if (originalThrowable != null && originalThrowable != t) =>
-          originalThrowable.addSuppressed(t)
-          logWarning(s"Suppressing exception in finally: ${t.getMessage}", t)
-          throw originalThrowable
-      }
-    }
-  }
-
   def startServiceOnPort[T](
       startPort: Int,
       startService: Int => (T, Int),
@@ -305,6 +306,7 @@ object Utils extends Logging {
     (base + offset - 1024) % (65536 - 1024) + 1024
   }
 
+  @tailrec
   def isBindCollision(exception: Throwable): Boolean = {
     exception match {
       case e: BindException =>
@@ -312,8 +314,6 @@ object Utils extends Logging {
           return true
         }
         isBindCollision(e.getCause)
-      //      case e: MultiException =>
-      //        e.getThrowables.asScala.exists(isBindCollision)
       case e: NativeIoException =>
         (e.getMessage != null && e.getMessage.startsWith("bind() failed: ")) ||
           isBindCollision(e.getCause)
@@ -403,22 +403,14 @@ object Utils extends Logging {
 
   private var customHostname: Option[String] = sys.env.get("CELEBORN_LOCAL_HOSTNAME")
 
+  // for testing
   def setCustomHostname(hostname: String) {
-    // DEBUG code
     Utils.checkHost(hostname)
     customHostname = Some(hostname)
   }
 
-  def localCanonicalHostName: String = {
-    customHostname.getOrElse(localIpAddress.getCanonicalHostName)
-  }
-
-  def localHostName: String = {
-    customHostname.getOrElse(localIpAddress.getHostAddress)
-  }
-
-  def localHostNameForURI: String = {
-    customHostname.getOrElse(InetAddresses.toUriString(localIpAddress))
+  def localHostName(conf: CelebornConf): String = customHostname.getOrElse {
+    if (conf.bindPreferIP) localIpAddress.getHostAddress else localIpAddress.getCanonicalHostName
   }
 
   /**
@@ -449,7 +441,7 @@ object Utils extends Logging {
 
   // Typically, this will be of order of number of nodes in cluster
   // If not, we should change it to LRUCache or something.
-  private val hostPortParseResults = new ConcurrentHashMap[String, (String, Int)]()
+  private val hostPortParseResults = JavaUtils.newConcurrentHashMap[String, (String, Int)]()
 
   def parseHostPort(hostPort: String): (String, Int) = {
     // Check cache first.
@@ -499,9 +491,9 @@ object Utils extends Logging {
     val numThreads = defaultNumThreads(numUsableCores)
     conf.setIfMissing(s"celeborn.$module.io.serverThreads", numThreads.toString)
     conf.setIfMissing(s"celeborn.$module.io.clientThreads", numThreads.toString)
-    // TODO remove after releasing 0.2.0
-    conf.setIfMissing(s"rss.$module.io.serverThreads", numThreads.toString)
-    conf.setIfMissing(s"rss.$module.io.clientThreads", numThreads.toString)
+    if (TransportModuleConstants.PUSH_MODULE == module) {
+      conf.setIfMissing(s"celeborn.$module.io.numConnectionsPerPeer", numThreads.toString)
+    }
 
     new TransportConf(module, conf)
   }
@@ -536,11 +528,11 @@ object Utils extends Logging {
     new File(clazz.getProtectionDomain.getCodeSource.getLocation.toURI).getPath
   }
 
-  def loadDefaultRssProperties(conf: CelebornConf, filePath: String = null): String = {
+  def loadDefaultCelebornProperties(conf: CelebornConf, filePath: String = null): String = {
     val path = Option(filePath).getOrElse(getDefaultPropertiesFile())
     Option(path).foreach { confFile =>
       getPropertiesFromFile(confFile).filter { case (k, v) =>
-        k.startsWith("celeborn.") || k.startsWith("rss.")
+        k.startsWith("celeborn.")
       }.foreach { case (k, v) =>
         conf.setIfMissing(k, v)
         sys.props.getOrElseUpdate(k, v)
@@ -596,7 +588,7 @@ object Utils extends Logging {
 
     } catch {
       case e: IOException =>
-        throw new CelebornException(s"Failed when loading RSS properties from $filename", e)
+        throw new CelebornException(s"Failed when loading Celeborn properties from $filename", e)
     } finally {
       inReader.close()
     }
@@ -620,8 +612,8 @@ object Utils extends Logging {
     (partitionId, epoch)
   }
 
-  def makeReducerKey(applicationId: String, shuffleId: Int, partitionId: Int): String = {
-    s"$applicationId-$shuffleId-$partitionId"
+  def makeReducerKey(shuffleId: Int, partitionId: Int): String = {
+    s"shuffleId-$partitionId"
   }
 
   def makeMapKey(applicationId: String, shuffleId: Int, mapId: Int, attemptId: Int): String = {
@@ -678,11 +670,78 @@ object Utils extends Logging {
     readProcessStdout(process)
   }
 
+  /**
+   * Execute a command and return the process running the command.
+   */
+  def executeCommand(
+      command: Seq[String],
+      workingDir: File = new File("."),
+      extraEnvironment: Map[String, String] = Map.empty,
+      redirectStderr: Boolean = true): Process = {
+    val builder = new ProcessBuilder(command: _*).directory(workingDir)
+    val environment = builder.environment()
+    for ((key, value) <- extraEnvironment) {
+      environment.put(key, value)
+    }
+    val process = builder.start()
+    if (redirectStderr) {
+      val threadName = "redirect stderr for command " + command.head
+
+      def log(s: String): Unit = logInfo(s)
+
+      processStreamByLine(threadName, process.getErrorStream, log)
+    }
+    process
+  }
+
+  /**
+   * Execute a command and get its output, throwing an exception if it yields a code other than 0.
+   */
+  def executeAndGetOutput(
+      command: Seq[String],
+      workingDir: File = new File("."),
+      extraEnvironment: Map[String, String] = Map.empty,
+      redirectStderr: Boolean = true): String = {
+    val process = executeCommand(command, workingDir, extraEnvironment, redirectStderr)
+    val output = new StringBuilder
+    val threadName = "read stdout for " + command.head
+
+    def appendToOutput(s: String): Unit = output.append(s).append("\n")
+
+    val stdoutThread = processStreamByLine(threadName, process.getInputStream, appendToOutput)
+    val exitCode = process.waitFor()
+    stdoutThread.join() // Wait for it to finish reading output
+    if (exitCode != 0) {
+      logError(s"Process $command exited with code $exitCode: $output")
+      throw new CelebornException(s"Process $command exited with code $exitCode")
+    }
+    output.toString
+  }
+
+  /**
+   * Return and start a daemon thread that processes the content of the input stream line by line.
+   */
+  def processStreamByLine(
+      threadName: String,
+      inputStream: InputStream,
+      processLine: String => Unit): Thread = {
+    val t = new Thread(threadName) {
+      override def run(): Unit = {
+        for (line <- Source.fromInputStream(inputStream).getLines()) {
+          processLine(line)
+        }
+      }
+    }
+    t.setDaemon(true)
+    t.start()
+    t
+  }
+
   /**
    * Create a directory inside the given parent directory. The directory is guaranteed to be
    * newly created, and is not marked for automatic deletion.
    */
-  def createDirectory(root: String, namePrefix: String = "rss"): File = {
+  def createDirectory(root: String, namePrefix: String = "celeborn"): File = {
     var attempts = 0
     val maxAttempts = 10
     var dir: File = null
@@ -732,7 +791,7 @@ object Utils extends Logging {
   def getSlotsPerDisk(
       slots: WorkerResource): util.Map[WorkerInfo, util.Map[String, Integer]] = {
     val workerSlotsDistribution = new util.HashMap[WorkerInfo, util.Map[String, Integer]]()
-    slots.asScala.foreach { case (workerInfo, (masterPartitionLoc, slavePartitionLoc)) =>
+    slots.asScala.foreach { case (workerInfo, (primaryPartitionLoc, replicaPartitionLoc)) =>
       val diskSlotsMap = new util.HashMap[String, Integer]()
 
       def countSlotsByDisk(location: util.List[PartitionLocation]): Unit = {
@@ -746,8 +805,8 @@ object Utils extends Logging {
         })
       }
 
-      countSlotsByDisk(masterPartitionLoc)
-      countSlotsByDisk(slavePartitionLoc)
+      countSlotsByDisk(primaryPartitionLoc)
+      countSlotsByDisk(replicaPartitionLoc)
       workerSlotsDistribution.put(workerInfo, diskSlotsMap)
     }
     workerSlotsDistribution
@@ -851,7 +910,7 @@ object Utils extends Logging {
       case 8 =>
         StatusCode.PARTITION_NOT_FOUND
       case 9 =>
-        StatusCode.SLAVE_PARTITION_NOT_FOUND
+        StatusCode.REPLICA_PARTITION_NOT_FOUND
       case 10 =>
         StatusCode.DELETE_FILES_FAILED
       case 11 =>
@@ -859,7 +918,7 @@ object Utils extends Logging {
       case 12 =>
         StatusCode.REVIVE_FAILED
       case 13 =>
-        StatusCode.PUSH_DATA_FAILED
+        StatusCode.REPLICATE_DATA_FAILED
       case 14 =>
         StatusCode.NUM_MAPPER_ZERO
       case 15 =>
@@ -869,9 +928,9 @@ object Utils extends Logging {
       case 17 =>
         StatusCode.PUSH_DATA_FAIL_NON_CRITICAL_CAUSE
       case 18 =>
-        StatusCode.PUSH_DATA_FAIL_SLAVE
+        StatusCode.PUSH_DATA_WRITE_FAIL_REPLICA
       case 19 =>
-        StatusCode.PUSH_DATA_FAIL_MASTER
+        StatusCode.PUSH_DATA_WRITE_FAIL_PRIMARY
       case 20 =>
         StatusCode.PUSH_DATA_FAIL_PARTITION_NOT_FOUND
       case 21 =>
@@ -887,13 +946,33 @@ object Utils extends Logging {
       case 26 =>
         StatusCode.NO_AVAILABLE_WORKING_DIR
       case 27 =>
-        StatusCode.WORKER_IN_BLACKLIST
+        StatusCode.WORKER_EXCLUDED
       case 28 =>
-        StatusCode.UNKNOWN_WORKER
+        StatusCode.WORKER_UNKNOWN
       case 30 =>
-        StatusCode.PUSH_DATA_SUCCESS_MASTER_CONGESTED
+        StatusCode.PUSH_DATA_SUCCESS_PRIMARY_CONGESTED
       case 31 =>
-        StatusCode.PUSH_DATA_SUCCESS_SLAVE_CONGESTED
+        StatusCode.PUSH_DATA_SUCCESS_REPLICA_CONGESTED
+      case 38 =>
+        StatusCode.PUSH_DATA_CREATE_CONNECTION_FAIL_PRIMARY
+      case 39 =>
+        StatusCode.PUSH_DATA_CREATE_CONNECTION_FAIL_REPLICA
+      case 40 =>
+        StatusCode.PUSH_DATA_CONNECTION_EXCEPTION_PRIMARY
+      case 41 =>
+        StatusCode.PUSH_DATA_CONNECTION_EXCEPTION_REPLICA
+      case 42 =>
+        StatusCode.PUSH_DATA_TIMEOUT_PRIMARY
+      case 43 =>
+        StatusCode.PUSH_DATA_TIMEOUT_REPLICA
+      case 44 =>
+        StatusCode.PUSH_DATA_PRIMARY_WORKER_EXCLUDED
+      case 45 =>
+        StatusCode.PUSH_DATA_REPLICA_WORKER_EXCLUDED
+      case 46 =>
+        StatusCode.FETCH_DATA_TIMEOUT
+      case 47 =>
+        StatusCode.REVIVE_INITIALIZED
       case _ =>
         null
     }
@@ -926,6 +1005,7 @@ object Utils extends Logging {
       case 1 => DiskStatus.READ_OR_WRITE_FAILURE
       case 2 => DiskStatus.IO_HANG
       case 3 => DiskStatus.HIGH_DISK_USAGE
+      case 4 => DiskStatus.CRITICAL_ERROR
       case _ => null
     }
   }
@@ -941,9 +1021,10 @@ object Utils extends Logging {
   val SORTED_SUFFIX = ".sorted"
   val INDEX_SUFFIX = ".index"
   val SUFFIX_HDFS_WRITE_SUCCESS = ".success"
+  val COMPATIBLE_HDFS_REGEX = "^[a-zA-Z0-9]+://.*"
 
   def isHdfsPath(path: String): Boolean = {
-    path.startsWith("hdfs://")
+    path.matches(COMPATIBLE_HDFS_REGEX)
   }
 
   def getSortedFilePath(path: String): String = {
@@ -981,4 +1062,36 @@ object Utils extends Logging {
     }
   }
 
+  def checkedDownCast(value: Long): Int = {
+    val downCast = value.toInt
+    if (downCast.toLong != value) {
+      throw new IllegalArgumentException("Cannot downcast long value " + value + " to integer.")
+    }
+    downCast
+  }
+
+  @throws[IOException]
+  def checkFileIntegrity(fileChannel: FileChannel, length: Int): Unit = {
+    val remainingBytes = fileChannel.size - fileChannel.position
+    if (remainingBytes < length) {
+      logError(
+        s"File remaining bytes not not enough, remaining: ${remainingBytes}, wanted: ${length}.")
+      throw new RuntimeException(s"File is corrupted ${fileChannel}")
+    }
+  }
+
+  def getShortFormattedFileName(fileInfo: FileInfo): String = {
+    val parentFile = fileInfo.getFile.getParent
+    parentFile.substring(
+      parentFile.lastIndexOf("/"),
+      parentFile.length) + "/" + fileInfo.getFile.getName
+  }
+
+  def parseMetricLabels(label: String): (String, String) = {
+    val labelPart = label.split("=")
+    if (labelPart.size != 2) {
+      throw new IllegalArgumentException(s"Illegal metric extra labels: $label")
+    }
+    labelPart(0).trim -> labelPart(1).trim
+  }
 }
