@@ -31,16 +31,14 @@ import scala.Tuple2;
 
 import com.google.common.annotations.VisibleForTesting;
 import com.google.protobuf.InvalidProtocolBufferException;
-import com.google.protobuf.ServiceException;
 import org.apache.ratis.RaftConfigKeys;
 import org.apache.ratis.conf.RaftProperties;
 import org.apache.ratis.grpc.GrpcConfigKeys;
 import org.apache.ratis.netty.NettyConfigKeys;
 import org.apache.ratis.proto.RaftProtos;
 import org.apache.ratis.protocol.*;
-import org.apache.ratis.protocol.exceptions.LeaderNotReadyException;
-import org.apache.ratis.protocol.exceptions.NotLeaderException;
-import org.apache.ratis.protocol.exceptions.StateMachineException;
+import org.apache.ratis.protocol.exceptions.RaftException;
+import org.apache.ratis.rpc.CallId;
 import org.apache.ratis.rpc.RpcType;
 import org.apache.ratis.rpc.SupportedRpcType;
 import org.apache.ratis.server.RaftServer;
@@ -48,13 +46,13 @@ import org.apache.ratis.server.RaftServerConfigKeys;
 import org.apache.ratis.thirdparty.com.google.protobuf.ByteString;
 import org.apache.ratis.util.LifeCycle;
 import org.apache.ratis.util.SizeInBytes;
-import org.apache.ratis.util.StringUtils;
 import org.apache.ratis.util.TimeDuration;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
 import org.apache.celeborn.common.CelebornConf;
-import org.apache.celeborn.common.haclient.RssHARetryClient;
+import org.apache.celeborn.common.client.MasterClient;
+import org.apache.celeborn.common.exception.CelebornRuntimeException;
 import org.apache.celeborn.common.util.ThreadUtils;
 import org.apache.celeborn.service.deploy.master.clustermeta.ResourceProtos;
 import org.apache.celeborn.service.deploy.master.clustermeta.ResourceProtos.ResourceResponse;
@@ -91,6 +89,9 @@ public class HARaftServer {
   private final ReentrantReadWriteLock roleCheckLock = new ReentrantReadWriteLock();
   private Optional<RaftProtos.RaftPeerRole> cachedPeerRole = Optional.empty();
   private Optional<String> cachedLeaderPeerRpcEndpoint = Optional.empty();
+  private final CelebornConf conf;
+  private long workerTimeoutDeadline;
+  private long appTimeoutDeadline;
 
   /**
    * Returns an Master Ratis server.
@@ -115,7 +116,9 @@ public class HARaftServer {
     this.raftPeerId = localRaftPeerId;
     this.raftGroup = RaftGroup.valueOf(RAFT_GROUP_ID, raftPeers);
     this.masterStateMachine = getStateMachine();
+    this.conf = conf;
     RaftProperties serverProperties = newRaftProperties(conf);
+    setDeadlineTime(Integer.MAX_VALUE, Integer.MAX_VALUE); // for default
     this.server =
         RaftServer.newBuilder()
             .setServerId(this.raftPeerId)
@@ -193,11 +196,12 @@ public class HARaftServer {
   }
 
   public ResourceResponse submitRequest(ResourceProtos.ResourceRequest request)
-      throws ServiceException {
+      throws CelebornRuntimeException {
     String requestId = request.getRequestId();
-    Tuple2<String, Long> decoded = RssHARetryClient.decodeRequestId(requestId);
+    Tuple2<String, Long> decoded = MasterClient.decodeRequestId(requestId);
     if (decoded == null) {
-      throw new ServiceException("RequestId:" + requestId + " invalid, should be: uuid#callId.");
+      throw new CelebornRuntimeException(
+          "RequestId:" + requestId + " invalid, should be: uuid#callId.");
     }
     ClientId clientId = ClientId.valueOf(UUID.fromString(decoded._1));
     long callId = decoded._2;
@@ -215,51 +219,20 @@ public class HARaftServer {
     try {
       raftClientReply = server.submitClientRequestAsync(raftClientRequest).get();
     } catch (Exception ex) {
-      throw new ServiceException(ex.getMessage(), ex);
+      throw new CelebornRuntimeException(ex.getMessage(), ex);
     }
 
     if (!raftClientReply.isSuccess()) {
-      NotLeaderException notLeaderException = raftClientReply.getNotLeaderException();
-      if (notLeaderException != null) {
-        throw new ServiceException("Not leader!");
-      }
-
-      LeaderNotReadyException leaderNotReadyException =
-          raftClientReply.getLeaderNotReadyException();
-      if (leaderNotReadyException != null) {
-        throw new ServiceException("Not leader!");
-      }
-
-      StateMachineException stateMachineException = raftClientReply.getStateMachineException();
-      if (stateMachineException != null) {
-        ResourceResponse.Builder response =
-            ResourceResponse.newBuilder().setCmdType(request.getCmdType()).setSuccess(false);
-        if (stateMachineException.getCause() != null) {
-          response.setMessage(stateMachineException.getCause().getMessage());
-          response.setStatus(ResourceProtos.Status.INTERNAL_ERROR);
-        } else {
-          // Current Ratis is setting cause, this is an safer side check.
-          LOG.error("StateMachine exception cause is not set");
-          response.setStatus(ResourceProtos.Status.INTERNAL_ERROR);
-          response.setMessage(StringUtils.stringifyException(stateMachineException));
-        }
-
-        if (LOG.isDebugEnabled()) {
-          LOG.debug("Error while executing ratis request.", stateMachineException);
-        }
-        return response.build();
-      }
+      RaftException exception = raftClientReply.getException();
+      throw new CelebornRuntimeException(
+          exception == null ? "Encounter raft exception." : exception.getMessage());
     }
 
     try {
       byte[] bytes = raftClientReply.getMessage().getContent().toByteArray();
       return ResourceResponse.newBuilder(ResourceResponse.parseFrom(bytes)).build();
     } catch (InvalidProtocolBufferException ex) {
-      if (ex.getMessage() != null) {
-        throw new ServiceException(ex.getMessage(), ex);
-      } else {
-        throw new ServiceException(ex);
-      }
+      throw new CelebornRuntimeException(ex.getMessage(), ex);
     }
   }
 
@@ -344,6 +317,13 @@ public class HARaftServer {
     RaftServerConfigKeys.Rpc.setTimeoutMin(properties, rpcTimeoutMin);
     RaftServerConfigKeys.Rpc.setTimeoutMax(properties, rpcTimeoutMax);
 
+    TimeDuration firstElectionTimeoutMin =
+        TimeDuration.valueOf(conf.haMasterRatisFirstElectionTimeoutMin(), TimeUnit.SECONDS);
+    TimeDuration firstElectionTimeoutMax =
+        TimeDuration.valueOf(conf.haMasterRatisFristElectionTimeoutMax(), TimeUnit.SECONDS);
+    RaftServerConfigKeys.Rpc.setFirstElectionTimeoutMin(properties, firstElectionTimeoutMin);
+    RaftServerConfigKeys.Rpc.setFirstElectionTimeoutMax(properties, firstElectionTimeoutMax);
+
     // Set the number of maximum cached segments
     RaftServerConfigKeys.Log.setSegmentCacheNumMax(properties, 2);
 
@@ -480,6 +460,22 @@ public class HARaftServer {
   private void setServerRole(RaftProtos.RaftPeerRole currentRole, String leaderPeerRpcEndpoint) {
     this.roleCheckLock.writeLock().lock();
     try {
+      boolean leaderChanged = false;
+      if (RaftProtos.RaftPeerRole.LEADER == currentRole && !checkCachedPeerRoleIsLeader()) {
+        leaderChanged = true;
+        setDeadlineTime(conf.workerHeartbeatTimeout(), conf.appHeartbeatTimeoutMs());
+      } else if (RaftProtos.RaftPeerRole.LEADER != currentRole && checkCachedPeerRoleIsLeader()) {
+        leaderChanged = true;
+        setDeadlineTime(Integer.MAX_VALUE, Integer.MAX_VALUE); // for revoke
+      }
+
+      if (leaderChanged) {
+        LOG.warn(
+            "Raft Role changed, CurrentNode Role: {}, Leader: {}",
+            currentRole,
+            leaderPeerRpcEndpoint);
+      }
+
       this.cachedPeerRole = Optional.ofNullable(currentRole);
       this.cachedLeaderPeerRpcEndpoint = Optional.ofNullable(leaderPeerRpcEndpoint);
     } finally {
@@ -500,4 +496,38 @@ public class HARaftServer {
   public String getRpcEndpoint() {
     return this.rpcEndpoint;
   }
+
+  void stepDown() {
+    try {
+      TransferLeadershipRequest request =
+          new TransferLeadershipRequest(
+              clientId,
+              server.getId(),
+              raftGroup.getGroupId(),
+              CallId.getAndIncrement(),
+              null,
+              60_000);
+      RaftClientReply reply = server.transferLeadership(request);
+      if (reply.isSuccess()) {
+        LOG.info("Successfully step down leader {}.", server.getId());
+      } else {
+        LOG.warn("Step down leader failed!");
+      }
+    } catch (Exception e) {
+      LOG.warn("Step down leader failed!", e);
+    }
+  }
+
+  public void setDeadlineTime(long increaseWorkerTime, long increaseAppTime) {
+    this.workerTimeoutDeadline = System.currentTimeMillis() + increaseWorkerTime;
+    this.appTimeoutDeadline = System.currentTimeMillis() + increaseAppTime;
+  }
+
+  public long getWorkerTimeoutDeadline() {
+    return workerTimeoutDeadline;
+  }
+
+  public long getAppTimeoutDeadline() {
+    return appTimeoutDeadline;
+  }
 }
