@@ -23,21 +23,24 @@ import java.util
 import java.util.concurrent.{ConcurrentHashMap, ScheduledFuture, TimeUnit}
 
 import scala.collection.JavaConverters._
+import scala.collection.mutable
 import scala.util.Random
 
+import org.apache.hadoop.fs.{FileSystem, Path}
+
 import org.apache.celeborn.common.CelebornConf
-import org.apache.celeborn.common.haclient.RssHARetryClient
+import org.apache.celeborn.common.client.MasterClient
 import org.apache.celeborn.common.identity.UserIdentifier
 import org.apache.celeborn.common.internal.Logging
 import org.apache.celeborn.common.meta.{DiskInfo, WorkerInfo}
 import org.apache.celeborn.common.metrics.MetricsSystem
-import org.apache.celeborn.common.metrics.source.{JVMCPUSource, JVMSource, RPCSource}
+import org.apache.celeborn.common.metrics.source.{JVMCPUSource, JVMSource, ResourceConsumptionSource, SystemMiscSource}
 import org.apache.celeborn.common.protocol._
 import org.apache.celeborn.common.protocol.message.{ControlMessages, StatusCode}
 import org.apache.celeborn.common.protocol.message.ControlMessages._
 import org.apache.celeborn.common.quota.{QuotaManager, ResourceConsumption}
 import org.apache.celeborn.common.rpc._
-import org.apache.celeborn.common.util.{PbSerDeUtils, ThreadUtils, Utils}
+import org.apache.celeborn.common.util.{CelebornHadoopUtils, JavaUtils, PbSerDeUtils, ThreadUtils, Utils}
 import org.apache.celeborn.server.common.{HttpService, Service}
 import org.apache.celeborn.service.deploy.master.clustermeta.SingleMasterMetaManager
 import org.apache.celeborn.service.deploy.master.clustermeta.ha.{HAHelper, HAMasterMetaManager, MetaHandler}
@@ -52,7 +55,7 @@ private[celeborn] class Master(
   override def serviceName: String = Service.MASTER
 
   override val metricsSystem: MetricsSystem =
-    MetricsSystem.createMetricsSystem(serviceName, conf, MasterSource.ServletPath)
+    MetricsSystem.createMetricsSystem(serviceName, conf, MetricsSystem.SERVLET_PATH)
 
   override val rpcEnv: RpcEnv = RpcEnv.create(
     RpcNameConstants.MASTER_SYS,
@@ -90,24 +93,35 @@ private[celeborn] class Master(
     ThreadUtils.newDaemonSingleThreadScheduledExecutor("master-forward-message-thread")
   private var checkForWorkerTimeOutTask: ScheduledFuture[_] = _
   private var checkForApplicationTimeOutTask: ScheduledFuture[_] = _
+  private var checkForHDFSRemnantDirsTimeOutTask: ScheduledFuture[_] = _
   private val nonEagerHandler = ThreadUtils.newDaemonCachedThreadPool("master-noneager-handler", 64)
 
   // Config constants
   private val workerHeartbeatTimeoutMs = conf.workerHeartbeatTimeout
   private val appHeartbeatTimeoutMs = conf.appHeartbeatTimeoutMs
+  private val hdfsExpireDirsTimeoutMS = conf.hdfsExpireDirsTimeoutMS
+  private val hasHDFSStorage = conf.hasHDFSStorage
 
   private val quotaManager = QuotaManager.instantiate(conf)
+  private val masterResourceConsumptionInterval = conf.masterResourceConsumptionInterval
+  private val userResourceConsumptions =
+    JavaUtils.newConcurrentHashMap[UserIdentifier, (ResourceConsumption, Long)]()
 
   // States
   private def workersSnapShot: util.List[WorkerInfo] =
     statusSystem.workers.synchronized(new util.ArrayList[WorkerInfo](statusSystem.workers))
+  private def lostWorkersSnapshot: ConcurrentHashMap[WorkerInfo, java.lang.Long] =
+    statusSystem.workers.synchronized(JavaUtils.newConcurrentHashMap(statusSystem.lostWorkers))
+  private def shutdownWorkerSnapshot: util.List[WorkerInfo] =
+    statusSystem.workers.synchronized(new util.ArrayList[WorkerInfo](statusSystem.shutdownWorkers))
 
-  private def diskReserveSize = conf.diskReserveSize
+  private def diskReserveSize = conf.workerDiskReserveSize
 
-  private val slotsAssignLoadAwareDiskGroupNum = conf.slotsAssignLoadAwareDiskGroupNum
-  private val slotsAssignLoadAwareDiskGroupGradient = conf.slotsAssignLoadAwareDiskGroupGradient
-  private val loadAwareFlushTimeWeight = conf.slotsAssignLoadAwareFlushTimeWeight
-  private val loadAwareFetchTimeWeight = conf.slotsAssignLoadAwareFetchTimeWeight
+  private val slotsAssignLoadAwareDiskGroupNum = conf.masterSlotAssignLoadAwareDiskGroupNum
+  private val slotsAssignLoadAwareDiskGroupGradient =
+    conf.masterSlotAssignLoadAwareDiskGroupGradient
+  private val loadAwareFlushTimeWeight = conf.masterSlotAssignLoadAwareFlushTimeWeight
+  private val loadAwareFetchTimeWeight = conf.masterSlotAssignLoadAwareFetchTimeWeight
 
   private val estimatedPartitionSizeUpdaterInitialDelay =
     conf.estimatedPartitionSizeUpdaterInitialDelay
@@ -128,28 +142,30 @@ private[celeborn] class Master(
     estimatedPartitionSizeUpdaterInitialDelay,
     estimatedPartitionSizeForEstimationUpdateInterval,
     TimeUnit.MILLISECONDS)
-  private val slotsAssignPolicy = conf.slotsAssignPolicy
+  private val slotsAssignPolicy = conf.masterSlotAssignPolicy
 
   // init and register master metrics
-  val rpcSource = new RPCSource(conf, MetricsSystem.ROLE_MASTER)
+  val resourceConsumptionSource = new ResourceConsumptionSource(conf)
   private val masterSource = new MasterSource(conf)
-  masterSource.addGauge(
-    MasterSource.RegisteredShuffleCount,
-    _ => statusSystem.registeredShuffle.size())
-  // blacklist worker count
-  masterSource.addGauge(MasterSource.BlacklistedWorkerCount, _ => statusSystem.blacklist.size())
-  // worker count
-  masterSource.addGauge(MasterSource.WorkerCount, _ => statusSystem.workers.size())
-  masterSource.addGauge(MasterSource.PartitionSize, _ => statusSystem.estimatedPartitionSize)
-  // is master active under HA mode
-  masterSource.addGauge(MasterSource.IsActiveMaster, _ => isMasterActive)
+  private var hadoopFs: FileSystem = _
+  masterSource.addGauge(MasterSource.REGISTERED_SHUFFLE_COUNT) { () =>
+    statusSystem.registeredShuffle.size
+  }
+  masterSource.addGauge(MasterSource.EXCLUDED_WORKER_COUNT) { () =>
+    statusSystem.excludedWorkers.size
+  }
+  masterSource.addGauge(MasterSource.WORKER_COUNT) { () => statusSystem.workers.size }
+  masterSource.addGauge(MasterSource.LOST_WORKER_COUNT) { () => statusSystem.lostWorkers.size }
+  masterSource.addGauge(MasterSource.PARTITION_SIZE) { () => statusSystem.estimatedPartitionSize }
+  masterSource.addGauge(MasterSource.IS_ACTIVE_MASTER) { () => isMasterActive }
 
-  metricsSystem.registerSource(rpcSource)
+  metricsSystem.registerSource(resourceConsumptionSource)
   metricsSystem.registerSource(masterSource)
   metricsSystem.registerSource(new JVMSource(conf, MetricsSystem.ROLE_MASTER))
   metricsSystem.registerSource(new JVMCPUSource(conf, MetricsSystem.ROLE_MASTER))
+  metricsSystem.registerSource(new SystemMiscSource(conf, MetricsSystem.ROLE_MASTER))
 
-  rpcEnv.setupEndpoint(RpcNameConstants.MASTER_EP, this, Some(rpcSource))
+  rpcEnv.setupEndpoint(RpcNameConstants.MASTER_EP, this)
 
   // start threads to check timeout for workers and applications
   override def onStart(): Unit = {
@@ -172,18 +188,34 @@ private[celeborn] class Master(
       0,
       appHeartbeatTimeoutMs / 2,
       TimeUnit.MILLISECONDS)
+
+    if (hasHDFSStorage) {
+      checkForHDFSRemnantDirsTimeOutTask = forwardMessageThread.scheduleAtFixedRate(
+        new Runnable {
+          override def run(): Unit = Utils.tryLogNonFatalError {
+            self.send(CheckForHDFSExpiredDirsTimeout)
+          }
+        },
+        hdfsExpireDirsTimeoutMS,
+        hdfsExpireDirsTimeoutMS,
+        TimeUnit.MILLISECONDS)
+    }
+
   }
 
   override def onStop(): Unit = {
-    logInfo("Stopping RSS Master.")
+    logInfo("Stopping Celeborn Master.")
     if (checkForWorkerTimeOutTask != null) {
       checkForWorkerTimeOutTask.cancel(true)
     }
     if (checkForApplicationTimeOutTask != null) {
       checkForApplicationTimeOutTask.cancel(true)
     }
+    if (checkForHDFSRemnantDirsTimeOutTask != null) {
+      checkForHDFSRemnantDirsTimeOutTask.cancel(true)
+    }
     forwardMessageThread.shutdownNow()
-    logInfo("RSS Master is stopped.")
+    logInfo("Celeborn Master is stopped.")
   }
 
   override def onDisconnected(address: RpcAddress): Unit = {
@@ -192,13 +224,22 @@ private[celeborn] class Master(
   }
 
   def executeWithLeaderChecker[T](context: RpcCallContext, f: => T): Unit =
-    if (HAHelper.checkShouldProcess(context, statusSystem)) f
+    if (HAHelper.checkShouldProcess(context, statusSystem)) {
+      try {
+        f
+      } catch {
+        case e: Exception =>
+          HAHelper.sendFailure(context, HAHelper.getRatisServer(statusSystem), e)
+      }
+    }
 
   override def receive: PartialFunction[Any, Unit] = {
     case _: PbCheckForWorkerTimeout =>
       executeWithLeaderChecker(null, timeoutDeadWorkers())
     case CheckForApplicationTimeOut =>
       executeWithLeaderChecker(null, timeoutDeadApplications())
+    case CheckForHDFSExpiredDirsTimeout =>
+      executeWithLeaderChecker(null, checkAndCleanExpiredAppDirsOnHDFS())
     case pb: PbWorkerLost =>
       val host = pb.getHost
       val rpcPort = pb.getRpcPort
@@ -206,18 +247,31 @@ private[celeborn] class Master(
       val fetchPort = pb.getFetchPort
       val replicatePort = pb.getReplicatePort
       val requestId = pb.getRequestId
-      logDebug(s"Received worker lost $host:$rpcPort:$pushPort:$fetchPort.")
+      logDebug(s"Received worker lost $host:$rpcPort:$pushPort:$fetchPort:$replicatePort.")
       executeWithLeaderChecker(
         null,
         handleWorkerLost(null, host, rpcPort, pushPort, fetchPort, replicatePort, requestId))
   }
 
   override def receiveAndReply(context: RpcCallContext): PartialFunction[Any, Unit] = {
-    case HeartbeatFromApplication(appId, totalWritten, fileCount, requestId) =>
+    case HeartbeatFromApplication(
+          appId,
+          totalWritten,
+          fileCount,
+          needCheckedWorkerList,
+          requestId,
+          shouldResponse) =>
       logDebug(s"Received heartbeat from app $appId")
       executeWithLeaderChecker(
         context,
-        handleHeartbeatFromApplication(context, appId, totalWritten, fileCount, requestId))
+        handleHeartbeatFromApplication(
+          context,
+          appId,
+          totalWritten,
+          fileCount,
+          needCheckedWorkerList,
+          requestId,
+          shouldResponse))
 
     case pbRegisterWorker: PbRegisterWorker =>
       val requestId = pbRegisterWorker.getRequestId
@@ -247,7 +301,7 @@ private[celeborn] class Master(
           userResourceConsumption,
           requestId))
 
-    case requestSlots @ RequestSlots(_, _, _, _, _, _, _) =>
+    case requestSlots @ RequestSlots(_, _, _, _, _, _, _, _) =>
       logTrace(s"Received RequestSlots request $requestSlots.")
       executeWithLeaderChecker(context, handleRequestSlots(context, requestSlots))
 
@@ -286,7 +340,7 @@ private[celeborn] class Master(
           estimatedAppDiskUsage,
           requestId) =>
       logDebug(s"Received heartbeat from" +
-        s" worker $host:$rpcPort:$pushPort:$fetchPort with $disks.")
+        s" worker $host:$rpcPort:$pushPort:$fetchPort:$replicatePort with $disks.")
       executeWithLeaderChecker(
         context,
         handleHeartbeatFromWorker(
@@ -302,20 +356,34 @@ private[celeborn] class Master(
           estimatedAppDiskUsage,
           requestId))
 
-    case GetWorkerInfos =>
-      executeWithLeaderChecker(context, handleGetWorkerInfos(context))
-
     case ReportWorkerUnavailable(failedWorkers: util.List[WorkerInfo], requestId: String) =>
       executeWithLeaderChecker(
         context,
         handleReportNodeUnavailable(context, failedWorkers, requestId))
 
+    case pb: PbWorkerLost =>
+      val host = pb.getHost
+      val rpcPort = pb.getRpcPort
+      val pushPort = pb.getPushPort
+      val fetchPort = pb.getFetchPort
+      val replicatePort = pb.getReplicatePort
+      val requestId = pb.getRequestId
+      logInfo(s"Received worker lost $host:$rpcPort:$pushPort:$fetchPort:$replicatePort.")
+      executeWithLeaderChecker(
+        context,
+        handleWorkerLost(context, host, rpcPort, pushPort, fetchPort, replicatePort, requestId))
+
     case CheckQuota(userIdentifier) =>
       executeWithLeaderChecker(context, handleCheckQuota(userIdentifier, context))
   }
 
   private def timeoutDeadWorkers() {
     val currentTime = System.currentTimeMillis()
+    // Need increase timeout deadline to avoid long time leader election period
+    if (HAHelper.getWorkerTimeoutDeadline(statusSystem) > currentTime) {
+      return
+    }
+
     var ind = 0
     workersSnapShot.asScala.foreach { worker =>
       if (worker.lastHeartbeat < currentTime - workerHeartbeatTimeoutMs
@@ -328,7 +396,7 @@ private[celeborn] class Master(
           worker.pushPort,
           worker.fetchPort,
           worker.replicatePort,
-          RssHARetryClient.genRequestId()))
+          MasterClient.genRequestId()))
       }
       ind += 1
     }
@@ -336,10 +404,14 @@ private[celeborn] class Master(
 
   private def timeoutDeadApplications(): Unit = {
     val currentTime = System.currentTimeMillis()
+    // Need increase timeout deadline to avoid long time leader election period
+    if (HAHelper.getAppTimeoutDeadline(statusSystem) > currentTime) {
+      return
+    }
     statusSystem.appHeartbeatTime.keySet().asScala.foreach { key =>
       if (statusSystem.appHeartbeatTime.get(key) < currentTime - appHeartbeatTimeoutMs) {
         logWarning(s"Application $key timeout, trigger applicationLost event.")
-        val requestId = RssHARetryClient.genRequestId()
+        val requestId = MasterClient.genRequestId()
         var res = self.askSync[ApplicationLostResponse](ApplicationLost(key, requestId))
         var retry = 1
         while (res.status != StatusCode.SUCCESS && retry <= 3) {
@@ -387,11 +459,12 @@ private[celeborn] class Master(
     val expiredShuffleKeys = new util.HashSet[String]
     activeShuffleKeys.asScala.foreach { shuffleKey =>
       if (!statusSystem.registeredShuffle.contains(shuffleKey)) {
-        logWarning(s"Shuffle $shuffleKey expired on $host:$rpcPort:$pushPort:$fetchPort.")
+        logWarning(
+          s"Shuffle $shuffleKey expired on $host:$rpcPort:$pushPort:$fetchPort:$replicatePort.")
         expiredShuffleKeys.add(shuffleKey)
       }
     }
-    context.reply(HeartbeatResponse(expiredShuffleKeys, registered))
+    context.reply(HeartbeatFromWorkerResponse(expiredShuffleKeys, registered))
   }
 
   private def handleWorkerLost(
@@ -409,20 +482,17 @@ private[celeborn] class Master(
       fetchPort,
       replicatePort,
       new util.HashMap[String, DiskInfo](),
-      new ConcurrentHashMap[UserIdentifier, ResourceConsumption](),
-      null)
+      JavaUtils.newConcurrentHashMap[UserIdentifier, ResourceConsumption]())
     val worker: WorkerInfo = workersSnapShot
       .asScala
       .find(_ == targetWorker)
       .orNull
     if (worker == null) {
-      logWarning(s"Unknown worker $host:$rpcPort:$pushPort:$fetchPort" +
+      logWarning(s"Unknown worker $host:$rpcPort:$pushPort:$fetchPort:$replicatePort" +
         s" for WorkerLost handler!")
-      return
+    } else {
+      statusSystem.handleWorkerLost(host, rpcPort, pushPort, fetchPort, replicatePort, requestId)
     }
-
-    statusSystem.handleWorkerLost(host, rpcPort, pushPort, fetchPort, replicatePort, requestId)
-
     if (context != null) {
       context.reply(WorkerLostResponse(true))
     }
@@ -446,8 +516,7 @@ private[celeborn] class Master(
         fetchPort,
         replicatePort,
         disks,
-        userResourceConsumption,
-        null)
+        userResourceConsumption)
     if (workersSnapShot.contains(workerToRegister)) {
       logWarning(s"Receive RegisterWorker while worker" +
         s" ${workerToRegister.toString()} already exists, re-register.")
@@ -495,25 +564,28 @@ private[celeborn] class Master(
     val numReducers = requestSlots.partitionIdList.size()
     val shuffleKey = Utils.makeShuffleKey(requestSlots.applicationId, requestSlots.shuffleId)
 
+    val availableWorkers = workersAvailable()
     // offer slots
     val slots =
-      masterSource.sample(MasterSource.OfferSlotsTime, s"offerSlots-${Random.nextInt()}") {
+      masterSource.sample(MasterSource.OFFER_SLOTS_TIME, s"offerSlots-${Random.nextInt()}") {
         statusSystem.workers.synchronized {
-          if (slotsAssignPolicy == SlotsAssignPolicy.ROUNDROBIN) {
-            SlotsAllocator.offerSlotsRoundRobin(
-              workersNotBlacklisted(),
-              requestSlots.partitionIdList,
-              requestSlots.shouldReplicate)
-          } else {
+          if (slotsAssignPolicy == SlotsAssignPolicy.LOADAWARE && !hasHDFSStorage) {
             SlotsAllocator.offerSlotsLoadAware(
-              workersNotBlacklisted(),
+              availableWorkers,
               requestSlots.partitionIdList,
               requestSlots.shouldReplicate,
+              requestSlots.shouldRackAware,
               diskReserveSize,
               slotsAssignLoadAwareDiskGroupNum,
               slotsAssignLoadAwareDiskGroupGradient,
               loadAwareFlushTimeWeight,
               loadAwareFetchTimeWeight)
+          } else {
+            SlotsAllocator.offerSlotsRoundRobin(
+              availableWorkers,
+              requestSlots.partitionIdList,
+              requestSlots.shouldReplicate,
+              requestSlots.shouldRackAware)
           }
         }
       }
@@ -542,8 +614,8 @@ private[celeborn] class Master(
     logInfo(s"Offer slots successfully for $numReducers reducers of $shuffleKey" +
       s" on ${slots.size()} workers.")
 
-    val workersNotSelected = workersNotBlacklisted().asScala.filter(!slots.containsKey(_))
-    val offerSlotsExtraSize = Math.min(conf.slotsAssignExtraSlots, workersNotSelected.size)
+    val workersNotSelected = availableWorkers.asScala.filter(!slots.containsKey(_))
+    val offerSlotsExtraSize = Math.min(conf.masterSlotAssignExtraSlots, workersNotSelected.size)
     if (offerSlotsExtraSize > 0) {
       var index = Random.nextInt(workersNotSelected.size)
       (1 to offerSlotsExtraSize).foreach(_ => {
@@ -565,9 +637,7 @@ private[celeborn] class Master(
       workerIds: util.List[String],
       slots: util.List[util.Map[String, Integer]],
       requestId: String): Unit = {
-    val shuffleKey = Utils.makeShuffleKey(applicationId, shuffleId)
-    statusSystem.handleReleaseSlots(shuffleKey, workerIds, slots, requestId)
-    logInfo(s"Release all slots of $shuffleKey")
+    // For compatibility, ignore this message
     context.reply(ReleaseSlotsResponse(StatusCode.SUCCESS))
   }
 
@@ -583,24 +653,20 @@ private[celeborn] class Master(
   }
 
   def handleGetBlacklist(context: RpcCallContext, msg: GetBlacklist): Unit = {
-    msg.localBlacklist.removeAll(workersSnapShot)
+    msg.localExcludedWorkers.removeAll(workersSnapShot)
     context.reply(
       GetBlacklistResponse(
         StatusCode.SUCCESS,
-        new util.ArrayList(statusSystem.blacklist),
-        msg.localBlacklist))
-  }
-
-  private def handleGetWorkerInfos(context: RpcCallContext): Unit = {
-    context.reply(GetWorkerInfosResponse(StatusCode.SUCCESS, workersSnapShot.asScala: _*))
+        new util.ArrayList(statusSystem.excludedWorkers),
+        msg.localExcludedWorkers))
   }
 
   private def handleReportNodeUnavailable(
       context: RpcCallContext,
       failedWorkers: util.List[WorkerInfo],
       requestId: String): Unit = {
-    logInfo(s"Receive ReportNodeFailure $failedWorkers, current blacklist" +
-      s"${statusSystem.blacklist}")
+    logInfo(s"Receive ReportNodeFailure $failedWorkers, current excluded workers" +
+      s"${statusSystem.excludedWorkers}")
     statusSystem.handleReportWorkerUnavailable(failedWorkers, requestId)
     context.reply(OneWayMessageResponse)
   }
@@ -610,136 +676,202 @@ private[celeborn] class Master(
       override def run(): Unit = {
         statusSystem.handleAppLost(appId, requestId)
         logInfo(s"Removed application $appId")
+        if (hasHDFSStorage) {
+          checkAndCleanExpiredAppDirsOnHDFS(appId)
+        }
         context.reply(ApplicationLostResponse(StatusCode.SUCCESS))
       }
     })
   }
 
+  private def checkAndCleanExpiredAppDirsOnHDFS(expiredDir: String = ""): Unit = {
+    if (hadoopFs == null) {
+      hadoopFs = CelebornHadoopUtils.getHadoopFS(conf)
+    }
+    val hdfsWorkPath = new Path(conf.hdfsDir, conf.workerWorkingDir)
+    if (hadoopFs.exists(hdfsWorkPath)) {
+      if (!expiredDir.isEmpty) {
+        val dirToDelete = new Path(hdfsWorkPath, expiredDir)
+        // delete specific app dir on application lost
+        CelebornHadoopUtils.deleteHDFSPathOrLogError(hadoopFs, dirToDelete, true)
+      } else {
+        val iter = hadoopFs.listStatusIterator(hdfsWorkPath)
+        while (iter.hasNext && isMasterActive == 1) {
+          val fileStatus = iter.next()
+          if (!statusSystem.appHeartbeatTime.containsKey(fileStatus.getPath.getName)) {
+            CelebornHadoopUtils.deleteHDFSPathOrLogError(hadoopFs, fileStatus.getPath, true)
+          }
+        }
+      }
+    }
+  }
+
   private def handleHeartbeatFromApplication(
       context: RpcCallContext,
       appId: String,
       totalWritten: Long,
       fileCount: Long,
-      requestId: String): Unit = {
+      needCheckedWorkerList: util.List[WorkerInfo],
+      requestId: String,
+      shouldResponse: Boolean): Unit = {
     statusSystem.handleAppHeartbeat(
       appId,
       totalWritten,
       fileCount,
       System.currentTimeMillis(),
       requestId)
-    context.reply(OneWayMessageResponse)
+    // unknown workers will retain in needCheckedWorkerList
+    needCheckedWorkerList.removeAll(workersSnapShot)
+    if (shouldResponse) {
+      context.reply(HeartbeatFromApplicationResponse(
+        StatusCode.SUCCESS,
+        new util.ArrayList(statusSystem.excludedWorkers),
+        needCheckedWorkerList,
+        shutdownWorkerSnapshot))
+    } else {
+      context.reply(OneWayMessageResponse)
+    }
+  }
+
+  private def computeUserResourceConsumption(userIdentifier: UserIdentifier)
+      : ResourceConsumption = {
+    val current = System.currentTimeMillis()
+    if (userResourceConsumptions.containsKey(userIdentifier)) {
+      val resourceConsumptionAndUpdateTime = userResourceConsumptions.get(userIdentifier)
+      if (current - resourceConsumptionAndUpdateTime._2 > masterResourceConsumptionInterval) {
+        val newResourceConsumption = statusSystem.workers.asScala.flatMap { workerInfo =>
+          workerInfo.userResourceConsumption.asScala.get(userIdentifier)
+        }.foldRight(ResourceConsumption(0, 0, 0, 0))(_ add _)
+        userResourceConsumptions.put(userIdentifier, (newResourceConsumption, current))
+        newResourceConsumption
+      } else {
+        resourceConsumptionAndUpdateTime._1
+      }
+    } else {
+      val newResourceConsumption = statusSystem.workers.asScala.flatMap { workerInfo =>
+        workerInfo.userResourceConsumption.asScala.get(userIdentifier)
+      }.foldRight(ResourceConsumption(0, 0, 0, 0))(_ add _)
+      userResourceConsumptions.put(userIdentifier, (newResourceConsumption, current))
+      newResourceConsumption
+    }
   }
 
   private def handleCheckQuota(
       userIdentifier: UserIdentifier,
       context: RpcCallContext): Unit = {
-    val userResourceConsumption = statusSystem.workers.asScala.flatMap { workerInfo =>
-      workerInfo.userResourceConsumption.asScala.get(userIdentifier)
-    }.foldRight(ResourceConsumption(0, 0, 0, 0))(_ add _)
+
+    resourceConsumptionSource.addGauge("diskFileCount", userIdentifier.toMap) { () =>
+      computeUserResourceConsumption(userIdentifier).diskFileCount
+    }
+    resourceConsumptionSource.addGauge("diskBytesWritten", userIdentifier.toMap) { () =>
+      computeUserResourceConsumption(userIdentifier).diskBytesWritten
+    }
+    resourceConsumptionSource.addGauge("hdfsFileCount", userIdentifier.toMap) { () =>
+      computeUserResourceConsumption(userIdentifier).hdfsFileCount
+    }
+    resourceConsumptionSource.addGauge("hdfsBytesWritten", userIdentifier.toMap) { () =>
+      computeUserResourceConsumption(userIdentifier).hdfsBytesWritten
+    }
+
+    val userResourceConsumption = computeUserResourceConsumption(userIdentifier)
     val quota = quotaManager.getQuota(userIdentifier)
-    val isAvailable = quota.checkQuotaSpaceAvailable(userIdentifier, userResourceConsumption)
-    context.reply(CheckQuotaResponse(isAvailable))
+    val (isAvailable, reason) =
+      quota.checkQuotaSpaceAvailable(userIdentifier, userResourceConsumption)
+    context.reply(CheckQuotaResponse(isAvailable, reason))
   }
 
-  private def workersNotBlacklisted(
-      tmpBlacklist: Set[WorkerInfo] = Set.empty): util.List[WorkerInfo] = {
+  private def workersAvailable(
+      tmpExcludedWorkerList: Set[WorkerInfo] = Set.empty): util.List[WorkerInfo] = {
     workersSnapShot.asScala.filter { w =>
-      !statusSystem.blacklist.contains(w) && !tmpBlacklist.contains(w)
+      !statusSystem.excludedWorkers.contains(w) && !statusSystem.shutdownWorkers.contains(
+        w) && !tmpExcludedWorkerList.contains(w)
     }.asJava
   }
 
   override def getWorkerInfo: String = {
     val sb = new StringBuilder
+    sb.append("====================== Workers Info in Master =========================\n")
     workersSnapShot.asScala.foreach { w =>
-      sb.append("==========WorkerInfos in Master==========\n")
       sb.append(w).append("\n")
-
-      val workerInfo = requestGetWorkerInfos(w.endpoint)
-        .workerInfos.asJava
-        .get(0)
-
-      sb.append("==========WorkerInfos in Workers==========\n")
-      sb.append(workerInfo).append("\n")
-
-      if (w.hasSameInfoWith(workerInfo)) {
-        sb.append("Consist!").append("\n")
-      } else {
-        sb.append("[ERROR] Inconsistent!").append("\n")
-      }
     }
+    sb.toString()
+  }
 
+  override def getLostWorkers: String = {
+    val sb = new StringBuilder
+    sb.append("======================= Lost Workers in Master ========================\n")
+    lostWorkersSnapshot.asScala.toSeq.sortBy(_._2).foreach { case (worker, time) =>
+      sb.append(s"${worker.toUniqueId().padTo(50, " ").mkString}${dateFmt.format(time)}\n")
+    }
+    sb.toString()
+  }
+
+  override def getShutdownWorkers: String = {
+    val sb = new StringBuilder
+    sb.append("===================== Shutdown Workers in Master ======================\n")
+    shutdownWorkerSnapshot.asScala.foreach { worker =>
+      sb.append(s"${worker.toUniqueId()}\n")
+    }
+    sb.toString()
+  }
+
+  override def getExcludedWorkers: String = {
+    val sb = new StringBuilder
+    sb.append("===================== Excluded Workers in Master ======================\n")
+    statusSystem.excludedWorkers.asScala.foreach { worker =>
+      sb.append(s"${worker.toUniqueId()}\n")
+    }
     sb.toString()
   }
 
   override def getThreadDump: String = {
     val sb = new StringBuilder
-    val threadDump = Utils.getThreadDump()
-    sb.append("==========Master ThreadDump==========\n")
-    sb.append(threadDump).append("\n")
-    workersSnapShot.asScala.foreach(w => {
-      sb.append(s"==========Worker ${w.readableAddress()} ThreadDump==========\n")
-      if (w.endpoint == null) {
-        w.setupEndpoint(this.rpcEnv.setupEndpointRef(
-          RpcAddress
-            .apply(w.host, w.rpcPort),
-          RpcNameConstants.WORKER_EP))
-      }
-      val res = requestThreadDump(w.endpoint)
-      sb.append(res.threadDump).append("\n")
-    })
-
+    sb.append("========================= Master ThreadDump ==========================\n")
+    sb.append(Utils.getThreadDump()).append("\n")
     sb.toString()
   }
 
   override def getHostnameList: String = {
-    statusSystem.hostnameSet.asScala.mkString("\n")
+    val sb = new StringBuilder
+    sb.append("================= LifecycleManager Hostname List ======================\n")
+    statusSystem.hostnameSet.asScala.foreach { host =>
+      sb.append(s"$host\n")
+    }
+    sb.toString()
   }
 
   override def getApplicationList: String = {
-    statusSystem.appHeartbeatTime.keys().asScala.mkString("\n")
+    val sb = new StringBuilder
+    sb.append("================= LifecycleManager Hostname List ======================\n")
+    statusSystem.appHeartbeatTime.asScala.toSeq.sortBy(_._2).foreach { case (appId, time) =>
+      sb.append(s"${appId.padTo(40, " ").mkString}${dateFmt.format(time)}\n")
+    }
+    sb.toString()
   }
 
   override def getShuffleList: String = {
-    statusSystem.registeredShuffle.asScala.mkString("\n")
+    val sb = new StringBuilder
+    sb.append("======================= Shuffle Key List ============================\n")
+    statusSystem.registeredShuffle.asScala.foreach { shuffleKey =>
+      sb.append(s"$shuffleKey\n")
+    }
+    sb.toString()
   }
 
   override def listTopDiskUseApps: String = {
-    statusSystem.appDiskUsageMetric.summary
+    val sb = new StringBuilder
+    sb.append("================== Top Disk Usage Applications =======================\n")
+    sb.append(statusSystem.appDiskUsageMetric.summary)
+    sb.toString()
   }
 
-  private def requestGetWorkerInfos(endpoint: RpcEndpointRef): GetWorkerInfosResponse = {
-    try {
-      if (endpoint != null) {
-        return endpoint.askSync[GetWorkerInfosResponse](GetWorkerInfos)
-      }
-    } catch {
-      case e: Exception =>
-        logError(s"AskSync GetWorkerInfos failed.", e)
-    }
-    val result = new util.ArrayList[WorkerInfo]
-    result.add(new WorkerInfo(
-      "unknown",
-      -1,
-      -1,
-      -1,
-      -1,
-      new util.HashMap[String, DiskInfo](),
-      new ConcurrentHashMap[UserIdentifier, ResourceConsumption](),
-      null))
-    GetWorkerInfosResponse(StatusCode.REQUEST_FAILED, result.asScala: _*)
-  }
+  override def listPartitionLocationInfo: String = throw new UnsupportedOperationException()
 
-  private def requestThreadDump(endpoint: RpcEndpointRef): ThreadDumpResponse = {
-    try {
-      if (endpoint != null) {
-        return endpoint.askSync[ThreadDumpResponse](ThreadDump)
-      }
-    } catch {
-      case e: Exception =>
-        logError(s"AskSync ThreadDump failed.", e)
-    }
-    ThreadDumpResponse("Unknown")
-  }
+  override def getUnavailablePeers: String = throw new UnsupportedOperationException()
+
+  override def isShutdown: String = throw new UnsupportedOperationException()
+
+  override def isRegistered: String = throw new UnsupportedOperationException()
 
   private def isMasterActive: Int = {
     // use int rather than bool for better monitoring on dashboard
