@@ -30,7 +30,7 @@ import org.junit.Before;
 import org.junit.Test;
 
 import org.apache.celeborn.common.CelebornConf;
-import org.apache.celeborn.common.haclient.RssHARetryClient;
+import org.apache.celeborn.common.client.MasterClient;
 import org.apache.celeborn.common.identity.UserIdentifier;
 import org.apache.celeborn.common.meta.DiskInfo;
 import org.apache.celeborn.common.meta.WorkerInfo;
@@ -105,8 +105,7 @@ public class DefaultMetaSystemSuiteJ {
   public void tearDown() throws Exception {}
 
   private String getNewReqeustId() {
-    return RssHARetryClient.encodeRequestId(
-        UUID.randomUUID().toString(), callerId.incrementAndGet());
+    return MasterClient.encodeRequestId(UUID.randomUUID().toString(), callerId.incrementAndGet());
   }
 
   @Test
@@ -220,8 +219,7 @@ public class DefaultMetaSystemSuiteJ {
             FETCHPORT1,
             REPLICATEPORT1,
             disks1,
-            userResourceConsumption1,
-            dummyRef);
+            userResourceConsumption1);
     WorkerInfo workerInfo2 =
         new WorkerInfo(
             HOSTNAME2,
@@ -230,8 +228,7 @@ public class DefaultMetaSystemSuiteJ {
             FETCHPORT2,
             REPLICATEPORT2,
             disks2,
-            userResourceConsumption2,
-            dummyRef);
+            userResourceConsumption2);
     WorkerInfo workerInfo3 =
         new WorkerInfo(
             HOSTNAME3,
@@ -240,8 +237,7 @@ public class DefaultMetaSystemSuiteJ {
             FETCHPORT3,
             REPLICATEPORT3,
             disks3,
-            userResourceConsumption3,
-            dummyRef);
+            userResourceConsumption3);
 
     Map<String, Map<String, Integer>> workersToAllocate = new HashMap<>();
     Map<String, Integer> allocation = new HashMap<>();
@@ -251,8 +247,8 @@ public class DefaultMetaSystemSuiteJ {
 
     statusSystem.handleRequestSlots(SHUFFLEKEY1, HOSTNAME1, workersToAllocate, getNewReqeustId());
 
-    assert (workerInfo1.usedSlots() == 5);
-    assert (workerInfo2.usedSlots() == 5);
+    assert (workerInfo1.usedSlots() == 0);
+    assert (workerInfo2.usedSlots() == 0);
     assert (workerInfo3.usedSlots() == 0);
   }
 
@@ -323,7 +319,7 @@ public class DefaultMetaSystemSuiteJ {
     statusSystem.handleReleaseSlots(SHUFFLEKEY1, workerIds, workerSlots, getNewReqeustId());
 
     Assert.assertEquals(
-        2,
+        0,
         statusSystem.workers.stream()
             .filter(w -> w.host().equals(HOSTNAME1))
             .findFirst()
@@ -369,8 +365,7 @@ public class DefaultMetaSystemSuiteJ {
             FETCHPORT1,
             REPLICATEPORT1,
             disks1,
-            userResourceConsumption1,
-            dummyRef);
+            userResourceConsumption1);
     WorkerInfo workerInfo2 =
         new WorkerInfo(
             HOSTNAME2,
@@ -379,8 +374,7 @@ public class DefaultMetaSystemSuiteJ {
             FETCHPORT2,
             REPLICATEPORT2,
             disks2,
-            userResourceConsumption2,
-            dummyRef);
+            userResourceConsumption2);
 
     Map<String, Map<String, Integer>> workersToAllocate = new HashMap<>();
     Map<String, Integer> allocation = new HashMap<>();
@@ -435,8 +429,7 @@ public class DefaultMetaSystemSuiteJ {
             FETCHPORT1,
             REPLICATEPORT1,
             disks1,
-            userResourceConsumption1,
-            dummyRef);
+            userResourceConsumption1);
     WorkerInfo workerInfo2 =
         new WorkerInfo(
             HOSTNAME2,
@@ -445,8 +438,7 @@ public class DefaultMetaSystemSuiteJ {
             FETCHPORT2,
             REPLICATEPORT2,
             disks2,
-            userResourceConsumption2,
-            dummyRef);
+            userResourceConsumption2);
 
     Map<String, Map<String, Integer>> workersToAllocate = new HashMap<>();
     Map<String, Integer> allocation = new HashMap<>();
@@ -518,7 +510,7 @@ public class DefaultMetaSystemSuiteJ {
         1,
         getNewReqeustId());
 
-    Assert.assertEquals(statusSystem.blacklist.size(), 1);
+    Assert.assertEquals(statusSystem.excludedWorkers.size(), 1);
 
     statusSystem.handleWorkerHeartbeat(
         HOSTNAME2,
@@ -532,7 +524,7 @@ public class DefaultMetaSystemSuiteJ {
         1,
         getNewReqeustId());
 
-    Assert.assertEquals(statusSystem.blacklist.size(), 2);
+    Assert.assertEquals(statusSystem.excludedWorkers.size(), 2);
 
     statusSystem.handleWorkerHeartbeat(
         HOSTNAME1,
@@ -546,7 +538,7 @@ public class DefaultMetaSystemSuiteJ {
         1,
         getNewReqeustId());
 
-    Assert.assertEquals(statusSystem.blacklist.size(), 2);
+    Assert.assertEquals(statusSystem.excludedWorkers.size(), 2);
   }
 
   @Test
@@ -587,8 +579,7 @@ public class DefaultMetaSystemSuiteJ {
             FETCHPORT1,
             REPLICATEPORT1,
             disks1,
-            userResourceConsumption1,
-            dummyRef);
+            userResourceConsumption1);
     WorkerInfo workerInfo2 =
         new WorkerInfo(
             HOSTNAME2,
@@ -597,14 +588,14 @@ public class DefaultMetaSystemSuiteJ {
             FETCHPORT2,
             REPLICATEPORT2,
             disks2,
-            userResourceConsumption2,
-            dummyRef);
+            userResourceConsumption2);
 
     List<WorkerInfo> failedWorkers = new ArrayList<>();
     failedWorkers.add(workerInfo1);
 
     statusSystem.handleReportWorkerUnavailable(failedWorkers, getNewReqeustId());
-    assert 1 == statusSystem.blacklist.size();
+    assert 1 == statusSystem.shutdownWorkers.size();
+    assert 0 == statusSystem.excludedWorkers.size();
   }
 
   @Test
