@@ -25,14 +25,12 @@ import java.io.IOException;
 import java.util.*;
 import java.util.concurrent.atomic.AtomicLong;
 
-import org.junit.Assert;
-import org.junit.Before;
-import org.junit.BeforeClass;
-import org.junit.Test;
+import org.junit.*;
 import org.mockito.Mockito;
 
 import org.apache.celeborn.common.CelebornConf;
-import org.apache.celeborn.common.haclient.RssHARetryClient;
+import org.apache.celeborn.common.client.MasterClient;
+import org.apache.celeborn.common.exception.CelebornRuntimeException;
 import org.apache.celeborn.common.identity.UserIdentifier;
 import org.apache.celeborn.common.meta.DiskInfo;
 import org.apache.celeborn.common.meta.WorkerInfo;
@@ -61,10 +59,26 @@ public class RatisMasterStatusSystemSuiteJ {
 
   @BeforeClass
   public static void init() throws IOException, InterruptedException {
+    resetRaftServer();
+  }
+
+  public static void resetRaftServer() throws IOException, InterruptedException {
     Mockito.when(mockRpcEnv.setupEndpointRef(Mockito.any(), Mockito.any()))
         .thenReturn(mockRpcEndpoint);
     when(mockRpcEnv.setupEndpointRef(any(), any())).thenReturn(dummyRef);
 
+    if (RATISSERVER1 != null) {
+      RATISSERVER1.stop();
+    }
+
+    if (RATISSERVER2 != null) {
+      RATISSERVER2.stop();
+    }
+
+    if (RATISSERVER3 != null) {
+      RATISSERVER3.stop();
+    }
+
     STATUSSYSTEM1 = new HAMasterMetaManager(mockRpcEnv, new CelebornConf());
     STATUSSYSTEM2 = new HAMasterMetaManager(mockRpcEnv, new CelebornConf());
     STATUSSYSTEM3 = new HAMasterMetaManager(mockRpcEnv, new CelebornConf());
@@ -74,22 +88,22 @@ public class RatisMasterStatusSystemSuiteJ {
     MetaHandler handler3 = new MetaHandler(STATUSSYSTEM3);
 
     CelebornConf conf1 = new CelebornConf();
-    File tmpDir1 = File.createTempFile("rss-ratis1", "for-test-only");
+    File tmpDir1 = File.createTempFile("celeborn-ratis1", "for-test-only");
     tmpDir1.delete();
     tmpDir1.mkdirs();
-    conf1.set("celeborn.ha.master.ratis.raft.server.storage.dir", tmpDir1.getAbsolutePath());
+    conf1.set(CelebornConf.HA_MASTER_RATIS_STORAGE_DIR().key(), tmpDir1.getAbsolutePath());
 
     CelebornConf conf2 = new CelebornConf();
-    File tmpDir2 = File.createTempFile("rss-ratis2", "for-test-only");
+    File tmpDir2 = File.createTempFile("celeborn-ratis2", "for-test-only");
     tmpDir2.delete();
     tmpDir2.mkdirs();
-    conf2.set("celeborn.ha.master.ratis.raft.server.storage.dir", tmpDir2.getAbsolutePath());
+    conf2.set(CelebornConf.HA_MASTER_RATIS_STORAGE_DIR().key(), tmpDir2.getAbsolutePath());
 
     CelebornConf conf3 = new CelebornConf();
-    File tmpDir3 = File.createTempFile("rss-ratis3", "for-test-only");
+    File tmpDir3 = File.createTempFile("celeborn-ratis3", "for-test-only");
     tmpDir3.delete();
     tmpDir3.mkdirs();
-    conf3.set("celeborn.ha.master.ratis.raft.server.storage.dir", tmpDir3.getAbsolutePath());
+    conf3.set(CelebornConf.HA_MASTER_RATIS_STORAGE_DIR().key(), tmpDir3.getAbsolutePath());
 
     String id1 = UUID.randomUUID().toString();
     String id2 = UUID.randomUUID().toString();
@@ -98,25 +112,23 @@ public class RatisMasterStatusSystemSuiteJ {
     int ratisPort2 = 9873;
     int ratisPort3 = 9874;
 
-    String localHost = Utils.localHostName();
-
     MasterNode masterNode1 =
         new MasterNode.Builder()
-            .setHost(localHost)
+            .setHost(Utils.localHostName(conf1))
             .setRatisPort(ratisPort1)
             .setRpcPort(9872)
             .setNodeId(id1)
             .build();
     MasterNode masterNode2 =
         new MasterNode.Builder()
-            .setHost(localHost)
+            .setHost(Utils.localHostName(conf2))
             .setRatisPort(ratisPort2)
             .setRpcPort(9873)
             .setNodeId(id2)
             .build();
     MasterNode masterNode3 =
         new MasterNode.Builder()
-            .setHost(localHost)
+            .setHost(Utils.localHostName(conf3))
             .setRatisPort(ratisPort3)
             .setRpcPort(9874)
             .setNodeId(id3)
@@ -183,8 +195,7 @@ public class RatisMasterStatusSystemSuiteJ {
   private static String SHUFFLEKEY1 = APPID1 + "-" + SHUFFLEID1;
 
   private String getNewReqeustId() {
-    return RssHARetryClient.encodeRequestId(
-        UUID.randomUUID().toString(), callerId.incrementAndGet());
+    return MasterClient.encodeRequestId(UUID.randomUUID().toString(), callerId.incrementAndGet());
   }
 
   public HAMasterMetaManager pickLeaderStatusSystem() {
@@ -200,6 +211,37 @@ public class RatisMasterStatusSystemSuiteJ {
     return null;
   }
 
+  private void stopNoneLeaderRaftServer(HARaftServer... raftServers) {
+    for (HARaftServer raftServer : raftServers) {
+      if (!raftServer.isLeader()) {
+        raftServer.stop();
+      }
+    }
+  }
+
+  @Test
+  public void testRaftSystemException() throws Exception {
+    AbstractMetaManager statusSystem = pickLeaderStatusSystem();
+    Assert.assertNotNull(statusSystem);
+    try {
+      stopNoneLeaderRaftServer(RATISSERVER1, RATISSERVER2, RATISSERVER3);
+      statusSystem.handleRegisterWorker(
+          HOSTNAME1,
+          RPCPORT1,
+          PUSHPORT1,
+          FETCHPORT1,
+          REPLICATEPORT1,
+          disks1,
+          userResourceConsumption1,
+          getNewReqeustId());
+      Assert.assertTrue(false);
+    } catch (CelebornRuntimeException e) {
+      Assert.assertTrue(true);
+    } finally {
+      resetRaftServer();
+    }
+  }
+
   @Test
   public void testHandleRegisterWorker() throws InterruptedException {
     AbstractMetaManager statusSystem = pickLeaderStatusSystem();
@@ -323,8 +365,7 @@ public class RatisMasterStatusSystemSuiteJ {
             FETCHPORT1,
             REPLICATEPORT1,
             disks1,
-            userResourceConsumption1,
-            dummyRef);
+            userResourceConsumption1);
     WorkerInfo workerInfo2 =
         new WorkerInfo(
             HOSTNAME2,
@@ -333,8 +374,7 @@ public class RatisMasterStatusSystemSuiteJ {
             FETCHPORT2,
             REPLICATEPORT2,
             disks2,
-            userResourceConsumption2,
-            dummyRef);
+            userResourceConsumption2);
     WorkerInfo workerInfo3 =
         new WorkerInfo(
             HOSTNAME3,
@@ -343,8 +383,7 @@ public class RatisMasterStatusSystemSuiteJ {
             FETCHPORT3,
             REPLICATEPORT3,
             disks3,
-            userResourceConsumption3,
-            dummyRef);
+            userResourceConsumption3);
 
     Map<String, Map<String, Integer>> workersToAllocate = new HashMap<>();
     Map<String, Integer> allocation1 = new HashMap<>();
@@ -359,22 +398,24 @@ public class RatisMasterStatusSystemSuiteJ {
 
     statusSystem.handleRequestSlots(SHUFFLEKEY1, HOSTNAME1, workersToAllocate, getNewReqeustId());
 
+    // Do not update diskinfo's activeslots
+
     Assert.assertEquals(
-        15,
+        0,
         statusSystem.workers.stream()
             .filter(w -> w.host().equals(HOSTNAME1))
             .findFirst()
             .get()
             .usedSlots());
     Assert.assertEquals(
-        25,
+        0,
         statusSystem.workers.stream()
             .filter(w -> w.host().equals(HOSTNAME2))
             .findFirst()
             .get()
             .usedSlots());
     Assert.assertEquals(
-        35,
+        0,
         statusSystem.workers.stream()
             .filter(w -> w.host().equals(HOSTNAME3))
             .findFirst()
@@ -456,22 +497,24 @@ public class RatisMasterStatusSystemSuiteJ {
     statusSystem.handleReleaseSlots(SHUFFLEKEY1, workerIds, workerSlots, getNewReqeustId());
     Thread.sleep(3000L);
 
+    // Do not update diskinfo's activeslots
+
     Assert.assertEquals(
-        2,
+        0,
         STATUSSYSTEM1.workers.stream()
             .filter(w -> w.host().equals(HOSTNAME1))
             .findFirst()
             .get()
             .usedSlots());
     Assert.assertEquals(
-        2,
+        0,
         STATUSSYSTEM2.workers.stream()
             .filter(w -> w.host().equals(HOSTNAME1))
             .findFirst()
             .get()
             .usedSlots());
     Assert.assertEquals(
-        2,
+        0,
         STATUSSYSTEM3.workers.stream()
             .filter(w -> w.host().equals(HOSTNAME1))
             .findFirst()
@@ -521,8 +564,7 @@ public class RatisMasterStatusSystemSuiteJ {
             FETCHPORT1,
             REPLICATEPORT1,
             disks1,
-            userResourceConsumption1,
-            dummyRef);
+            userResourceConsumption1);
     WorkerInfo workerInfo2 =
         new WorkerInfo(
             HOSTNAME2,
@@ -531,8 +573,7 @@ public class RatisMasterStatusSystemSuiteJ {
             FETCHPORT2,
             REPLICATEPORT2,
             disks2,
-            userResourceConsumption2,
-            dummyRef);
+            userResourceConsumption2);
     Map<String, Map<String, Integer>> workersToAllocate = new HashMap<>();
     Map<String, Integer> allocations = new HashMap<>();
     allocations.put("disk1", 5);
@@ -595,8 +636,7 @@ public class RatisMasterStatusSystemSuiteJ {
             FETCHPORT1,
             REPLICATEPORT1,
             disks1,
-            userResourceConsumption1,
-            dummyRef);
+            userResourceConsumption1);
     WorkerInfo workerInfo2 =
         new WorkerInfo(
             HOSTNAME2,
@@ -605,8 +645,7 @@ public class RatisMasterStatusSystemSuiteJ {
             FETCHPORT2,
             REPLICATEPORT2,
             disks2,
-            userResourceConsumption2,
-            dummyRef);
+            userResourceConsumption2);
 
     Map<String, Map<String, Integer>> workersToAllocate = new HashMap<>();
     Map<String, Integer> allocations = new HashMap<>();
@@ -700,9 +739,9 @@ public class RatisMasterStatusSystemSuiteJ {
         getNewReqeustId());
     Thread.sleep(3000L);
 
-    Assert.assertEquals(1, STATUSSYSTEM1.blacklist.size());
-    Assert.assertEquals(1, STATUSSYSTEM2.blacklist.size());
-    Assert.assertEquals(1, STATUSSYSTEM3.blacklist.size());
+    Assert.assertEquals(1, STATUSSYSTEM1.excludedWorkers.size());
+    Assert.assertEquals(1, STATUSSYSTEM2.excludedWorkers.size());
+    Assert.assertEquals(1, STATUSSYSTEM3.excludedWorkers.size());
 
     statusSystem.handleWorkerHeartbeat(
         HOSTNAME2,
@@ -717,10 +756,10 @@ public class RatisMasterStatusSystemSuiteJ {
         getNewReqeustId());
     Thread.sleep(3000L);
 
-    Assert.assertEquals(2, statusSystem.blacklist.size());
-    Assert.assertEquals(2, STATUSSYSTEM1.blacklist.size());
-    Assert.assertEquals(2, STATUSSYSTEM2.blacklist.size());
-    Assert.assertEquals(2, STATUSSYSTEM3.blacklist.size());
+    Assert.assertEquals(2, statusSystem.excludedWorkers.size());
+    Assert.assertEquals(2, STATUSSYSTEM1.excludedWorkers.size());
+    Assert.assertEquals(2, STATUSSYSTEM2.excludedWorkers.size());
+    Assert.assertEquals(2, STATUSSYSTEM3.excludedWorkers.size());
 
     statusSystem.handleWorkerHeartbeat(
         HOSTNAME1,
@@ -735,10 +774,10 @@ public class RatisMasterStatusSystemSuiteJ {
         getNewReqeustId());
     Thread.sleep(3000L);
 
-    Assert.assertEquals(1, statusSystem.blacklist.size());
-    Assert.assertEquals(1, STATUSSYSTEM1.blacklist.size());
-    Assert.assertEquals(1, STATUSSYSTEM2.blacklist.size());
-    Assert.assertEquals(1, STATUSSYSTEM3.blacklist.size());
+    Assert.assertEquals(1, statusSystem.excludedWorkers.size());
+    Assert.assertEquals(1, STATUSSYSTEM1.excludedWorkers.size());
+    Assert.assertEquals(1, STATUSSYSTEM2.excludedWorkers.size());
+    Assert.assertEquals(1, STATUSSYSTEM3.excludedWorkers.size());
   }
 
   @Before
@@ -747,21 +786,21 @@ public class RatisMasterStatusSystemSuiteJ {
     STATUSSYSTEM1.hostnameSet.clear();
     STATUSSYSTEM1.workers.clear();
     STATUSSYSTEM1.appHeartbeatTime.clear();
-    STATUSSYSTEM1.blacklist.clear();
+    STATUSSYSTEM1.excludedWorkers.clear();
     STATUSSYSTEM1.workerLostEvents.clear();
 
     STATUSSYSTEM2.registeredShuffle.clear();
     STATUSSYSTEM2.hostnameSet.clear();
     STATUSSYSTEM2.workers.clear();
     STATUSSYSTEM2.appHeartbeatTime.clear();
-    STATUSSYSTEM2.blacklist.clear();
+    STATUSSYSTEM2.excludedWorkers.clear();
     STATUSSYSTEM2.workerLostEvents.clear();
 
     STATUSSYSTEM3.registeredShuffle.clear();
     STATUSSYSTEM3.hostnameSet.clear();
     STATUSSYSTEM3.workers.clear();
     STATUSSYSTEM3.appHeartbeatTime.clear();
-    STATUSSYSTEM3.blacklist.clear();
+    STATUSSYSTEM3.excludedWorkers.clear();
     STATUSSYSTEM3.workerLostEvents.clear();
 
     disks1.clear();
@@ -824,8 +863,7 @@ public class RatisMasterStatusSystemSuiteJ {
             FETCHPORT1,
             REPLICATEPORT1,
             disks1,
-            userResourceConsumption1,
-            dummyRef);
+            userResourceConsumption1);
     WorkerInfo workerInfo2 =
         new WorkerInfo(
             HOSTNAME2,
@@ -834,17 +872,19 @@ public class RatisMasterStatusSystemSuiteJ {
             FETCHPORT2,
             REPLICATEPORT2,
             disks2,
-            userResourceConsumption2,
-            dummyRef);
+            userResourceConsumption2);
 
     List<WorkerInfo> failedWorkers = new ArrayList<>();
     failedWorkers.add(workerInfo1);
 
     statusSystem.handleReportWorkerUnavailable(failedWorkers, getNewReqeustId());
     Thread.sleep(3000L);
-    Assert.assertEquals(1, STATUSSYSTEM1.blacklist.size());
-    Assert.assertEquals(1, STATUSSYSTEM2.blacklist.size());
-    Assert.assertEquals(1, STATUSSYSTEM3.blacklist.size());
+    Assert.assertEquals(1, STATUSSYSTEM1.shutdownWorkers.size());
+    Assert.assertEquals(1, STATUSSYSTEM2.shutdownWorkers.size());
+    Assert.assertEquals(1, STATUSSYSTEM3.shutdownWorkers.size());
+    Assert.assertEquals(0, STATUSSYSTEM1.excludedWorkers.size());
+    Assert.assertEquals(0, STATUSSYSTEM2.excludedWorkers.size());
+    Assert.assertEquals(0, STATUSSYSTEM3.excludedWorkers.size());
   }
 
   @Test
@@ -855,4 +895,18 @@ public class RatisMasterStatusSystemSuiteJ {
     statusSystem.handleUpdatePartitionSize();
     Thread.sleep(3000L);
   }
+
+  @AfterClass
+  public static void testNotifyLogFailed() {
+    List<HARaftServer> list = Arrays.asList(RATISSERVER1, RATISSERVER2, RATISSERVER3);
+    for (HARaftServer haRaftServer : list) {
+      if (haRaftServer.isLeader()) {
+        haRaftServer
+            .getMasterStateMachine()
+            .notifyLogFailed(new Exception("test leader step down"), null);
+        Assert.assertFalse(haRaftServer.isLeader());
+        break;
+      }
+    }
+  }
 }
