@@ -18,8 +18,6 @@
 package org.apache.celeborn.service.deploy.worker.storage;
 
 import java.io.File;
-import java.io.FileInputStream;
-import java.io.FileOutputStream;
 import java.io.IOException;
 import java.nio.ByteBuffer;
 import java.nio.channels.FileChannel;
@@ -52,16 +50,13 @@ import org.apache.celeborn.common.CelebornConf;
 import org.apache.celeborn.common.identity.UserIdentifier;
 import org.apache.celeborn.common.meta.FileInfo;
 import org.apache.celeborn.common.metrics.source.AbstractSource;
-import org.apache.celeborn.common.network.server.memory.MemoryManager;
 import org.apache.celeborn.common.unsafe.Platform;
-import org.apache.celeborn.common.util.PbSerDeUtils;
-import org.apache.celeborn.common.util.ShuffleBlockInfoUtils;
+import org.apache.celeborn.common.util.*;
 import org.apache.celeborn.common.util.ShuffleBlockInfoUtils.ShuffleBlockInfo;
-import org.apache.celeborn.common.util.ThreadUtils;
-import org.apache.celeborn.common.util.Utils;
 import org.apache.celeborn.service.deploy.worker.LevelDBProvider;
 import org.apache.celeborn.service.deploy.worker.ShuffleRecoverHelper;
 import org.apache.celeborn.service.deploy.worker.WorkerSource;
+import org.apache.celeborn.service.deploy.worker.memory.MemoryManager;
 
 public class PartitionFilesSorter extends ShuffleRecoverHelper {
   private static final Logger logger = LoggerFactory.getLogger(PartitionFilesSorter.class);
@@ -72,11 +67,11 @@ public class PartitionFilesSorter extends ShuffleRecoverHelper {
   private File recoverFile;
   private volatile boolean shutdown = false;
   private final ConcurrentHashMap<String, Set<String>> sortedShuffleFiles =
-      new ConcurrentHashMap<>();
+      JavaUtils.newConcurrentHashMap();
   private final ConcurrentHashMap<String, Set<String>> sortingShuffleFiles =
-      new ConcurrentHashMap<>();
+      JavaUtils.newConcurrentHashMap();
   private final ConcurrentHashMap<String, Map<String, Map<Integer, List<ShuffleBlockInfo>>>>
-      cachedIndexMaps = new ConcurrentHashMap<>();
+      cachedIndexMaps = JavaUtils.newConcurrentHashMap();
   private final LinkedBlockingQueue<FileSorter> shuffleSortTaskDeque = new LinkedBlockingQueue<>();
 
   private final AtomicInteger sortedFileCount = new AtomicInteger();
@@ -91,11 +86,7 @@ public class PartitionFilesSorter extends ShuffleRecoverHelper {
 
   protected final AbstractSource source;
 
-  private final ExecutorService fileSorterExecutors =
-      ThreadUtils.newDaemonCachedThreadPool(
-          "worker-file-sorter-execute",
-          Math.max(Runtime.getRuntime().availableProcessors(), 8),
-          120);
+  private final ExecutorService fileSorterExecutors;
   private final Thread fileSorterSchedulerThread;
 
   public PartitionFilesSorter(
@@ -103,7 +94,8 @@ public class PartitionFilesSorter extends ShuffleRecoverHelper {
     this.sortTimeout = conf.partitionSorterSortPartitionTimeout();
     this.shuffleChunkSize = conf.shuffleChunkSize();
     this.reservedMemoryPerPartition = conf.partitionSorterReservedMemoryPerPartition();
-    this.partitionSorterShutdownAwaitTime = conf.partitionSorterCloseAwaitTimeMs();
+    this.partitionSorterShutdownAwaitTime =
+        conf.workerGracefulShutdownPartitionSorterCloseAwaitTimeMs();
     this.source = source;
     this.memoryManager = memoryManager;
     this.gracefulShutdown = conf.workerGracefulShutdown();
@@ -111,7 +103,7 @@ public class PartitionFilesSorter extends ShuffleRecoverHelper {
     // when the worker's fetching port is stable and enables graceful shutdown.
     if (gracefulShutdown) {
       try {
-        String recoverPath = conf.workerRecoverPath();
+        String recoverPath = conf.workerGracefulShutdownRecoverPath();
         this.recoverFile = new File(recoverPath, RECOVERY_SORTED_FILES_FILE_NAME);
         this.sortedFilesDb = LevelDBProvider.initLevelDB(recoverFile, CURRENT_VERSION);
         reloadAndCleanSortedShuffleFiles(this.sortedFilesDb);
@@ -123,6 +115,10 @@ public class PartitionFilesSorter extends ShuffleRecoverHelper {
       this.sortedFilesDb = null;
     }
 
+    fileSorterExecutors =
+        ThreadUtils.newDaemonCachedThreadPool(
+            "worker-file-sorter-execute", conf.partitionSorterThreads(), 120);
+
     fileSorterSchedulerThread =
         new Thread(
             () -> {
@@ -158,8 +154,8 @@ public class PartitionFilesSorter extends ShuffleRecoverHelper {
     return sortedFileCount.get();
   }
 
-  public int getSortedSize() {
-    return (int) sortedFilesSize.get();
+  public long getSortedSize() {
+    return sortedFilesSize.get();
   }
 
   public FileInfo getSortedFileInfo(
@@ -197,8 +193,8 @@ public class PartitionFilesSorter extends ShuffleRecoverHelper {
           throw new IOException(
               "Sort scheduler thread is interrupted means worker is shutting down.", e);
         } catch (IOException e) {
-          logger.error("File sorter access hdfs failed.", e);
-          throw new IOException("File sorter access hdfs failed.", e);
+          logger.error("File sorter access HDFS failed.", e);
+          throw new IOException("File sorter access HDFS failed.", e);
         }
       }
     }
@@ -341,9 +337,9 @@ public class PartitionFilesSorter extends ShuffleRecoverHelper {
     if (isHdfs) {
       // If the index file exists, it will be overwritten.
       // So there is no need to check its existence.
-      hdfsIndexOutput = StorageManager.hdfsFs().create(new Path(indexFilePath));
+      hdfsIndexOutput = StorageManager.hadoopFs().create(new Path(indexFilePath));
     } else {
-      indexFileChannel = new FileOutputStream(indexFilePath).getChannel();
+      indexFileChannel = FileChannelUtils.createWritableFileChannel(indexFilePath);
     }
 
     int indexSize = 0;
@@ -447,16 +443,17 @@ public class PartitionFilesSorter extends ShuffleRecoverHelper {
         && cachedIndexMaps.get(shuffleKey).containsKey(fileId)) {
       indexMap = cachedIndexMaps.get(shuffleKey).get(fileId);
     } else {
-      FileInputStream indexStream = null;
+      FileChannel indexChannel = null;
       FSDataInputStream hdfsIndexStream = null;
       boolean isHdfs = Utils.isHdfsPath(indexFilePath);
       int indexSize = 0;
       try {
         if (isHdfs) {
-          hdfsIndexStream = StorageManager.hdfsFs().open(new Path(indexFilePath));
-          indexSize = (int) StorageManager.hdfsFs().getFileStatus(new Path(indexFilePath)).getLen();
+          hdfsIndexStream = StorageManager.hadoopFs().open(new Path(indexFilePath));
+          indexSize =
+              (int) StorageManager.hadoopFs().getFileStatus(new Path(indexFilePath)).getLen();
         } else {
-          indexStream = new FileInputStream(indexFilePath);
+          indexChannel = FileChannelUtils.openReadableFileChannel(indexFilePath);
           File indexFile = new File(indexFilePath);
           indexSize = (int) indexFile.length();
         }
@@ -464,18 +461,18 @@ public class PartitionFilesSorter extends ShuffleRecoverHelper {
         if (isHdfs) {
           readStreamFully(hdfsIndexStream, indexBuf, indexFilePath);
         } else {
-          readChannelFully(indexStream.getChannel(), indexBuf, indexFilePath);
+          readChannelFully(indexChannel, indexBuf, indexFilePath);
         }
         indexBuf.rewind();
         indexMap = ShuffleBlockInfoUtils.parseShuffleBlockInfosFromByteBuffer(indexBuf);
         Map<String, Map<Integer, List<ShuffleBlockInfo>>> cacheMap =
-            cachedIndexMaps.computeIfAbsent(shuffleKey, v -> new ConcurrentHashMap<>());
+            cachedIndexMaps.computeIfAbsent(shuffleKey, v -> JavaUtils.newConcurrentHashMap());
         cacheMap.put(fileId, indexMap);
       } catch (Exception e) {
         logger.error("Read sorted shuffle file index " + indexFilePath + " error, detail: ", e);
         throw new IOException("Read sorted shuffle file index failed.", e);
       } finally {
-        IOUtils.closeQuietly(indexStream, null);
+        IOUtils.closeQuietly(indexChannel, null);
         IOUtils.closeQuietly(hdfsIndexStream, null);
       }
     }
@@ -518,17 +515,17 @@ public class PartitionFilesSorter extends ShuffleRecoverHelper {
           indexFile.delete();
         }
       } else {
-        if (StorageManager.hdfsFs().exists(fileInfo.getHdfsSortedPath())) {
-          StorageManager.hdfsFs().delete(fileInfo.getHdfsSortedPath(), false);
+        if (StorageManager.hadoopFs().exists(fileInfo.getHdfsSortedPath())) {
+          StorageManager.hadoopFs().delete(fileInfo.getHdfsSortedPath(), false);
         }
-        if (StorageManager.hdfsFs().exists(fileInfo.getHdfsIndexPath())) {
-          StorageManager.hdfsFs().delete(fileInfo.getHdfsIndexPath(), false);
+        if (StorageManager.hadoopFs().exists(fileInfo.getHdfsIndexPath())) {
+          StorageManager.hadoopFs().delete(fileInfo.getHdfsIndexPath(), false);
         }
       }
     }
 
     public void sort() throws InterruptedException {
-      source.startTimer(WorkerSource.SortTime(), fileId);
+      source.startTimer(WorkerSource.SORT_TIME(), fileId);
 
       try {
         initializeFiles();
@@ -597,16 +594,17 @@ public class PartitionFilesSorter extends ShuffleRecoverHelper {
           sorting.remove(fileId);
         }
       }
-      source.stopTimer(WorkerSource.SortTime(), fileId);
+      source.stopTimer(WorkerSource.SORT_TIME(), fileId);
     }
 
     private void initializeFiles() throws IOException {
       if (isHdfs) {
-        hdfsOriginInput = StorageManager.hdfsFs().open(new Path(originFilePath));
-        hdfsSortedOutput = StorageManager.hdfsFs().create(new Path(sortedFilePath));
+        hdfsOriginInput = StorageManager.hadoopFs().open(new Path(originFilePath));
+        hdfsSortedOutput =
+            StorageManager.hadoopFs().create(new Path(sortedFilePath), true, 256 * 1024);
       } else {
-        originFileChannel = new FileInputStream(originFilePath).getChannel();
-        sortedFileChannel = new FileOutputStream(sortedFilePath).getChannel();
+        originFileChannel = FileChannelUtils.openReadableFileChannel(originFilePath);
+        sortedFileChannel = FileChannelUtils.createWritableFileChannel(sortedFilePath);
       }
     }
 
@@ -636,7 +634,7 @@ public class PartitionFilesSorter extends ShuffleRecoverHelper {
     private void deleteOriginFiles() throws IOException {
       boolean deleteSuccess = false;
       if (isHdfs) {
-        deleteSuccess = StorageManager.hdfsFs().delete(new Path(originFilePath), false);
+        deleteSuccess = StorageManager.hadoopFs().delete(new Path(originFilePath), false);
       } else {
         deleteSuccess = new File(originFilePath).delete();
       }
