@@ -21,16 +21,14 @@ import java.io.{File, IOException}
 import java.nio.charset.StandardCharsets
 import java.nio.file.{FileAlreadyExistsException, Files, Paths}
 import java.util
-import java.util.concurrent.{ConcurrentHashMap, Executors, ThreadPoolExecutor, TimeUnit}
+import java.util.concurrent.{ConcurrentHashMap, ThreadPoolExecutor, TimeUnit}
 import java.util.concurrent.atomic.AtomicInteger
-import java.util.function.IntUnaryOperator
+import java.util.function.{BiConsumer, IntUnaryOperator}
 
 import scala.collection.JavaConverters._
-import scala.collection.mutable
 import scala.concurrent.duration._
 
-import com.google.common.util.concurrent.ThreadFactoryBuilder
-import org.apache.hadoop.conf.Configuration
+import io.netty.buffer.PooledByteBufAllocator
 import org.apache.hadoop.fs.{FileSystem, Path}
 import org.apache.hadoop.fs.permission.FsPermission
 import org.iq80.leveldb.DB
@@ -41,25 +39,30 @@ import org.apache.celeborn.common.identity.UserIdentifier
 import org.apache.celeborn.common.internal.Logging
 import org.apache.celeborn.common.meta.{DeviceInfo, DiskInfo, DiskStatus, FileInfo, TimeWindow}
 import org.apache.celeborn.common.metrics.source.AbstractSource
-import org.apache.celeborn.common.network.server.memory.MemoryManager.MemoryPressureListener
+import org.apache.celeborn.common.network.util.{NettyUtils, TransportConf}
 import org.apache.celeborn.common.protocol.{PartitionLocation, PartitionSplitMode, PartitionType}
 import org.apache.celeborn.common.quota.ResourceConsumption
-import org.apache.celeborn.common.util.{PbSerDeUtils, ThreadUtils, Utils}
+import org.apache.celeborn.common.util.{CelebornHadoopUtils, JavaUtils, PbSerDeUtils, ThreadUtils, Utils}
 import org.apache.celeborn.service.deploy.worker._
-import org.apache.celeborn.service.deploy.worker.storage.StorageManager.hdfsFs
+import org.apache.celeborn.service.deploy.worker.memory.MemoryManager.MemoryPressureListener
+import org.apache.celeborn.service.deploy.worker.storage.StorageManager.hadoopFs
 
 final private[worker] class StorageManager(conf: CelebornConf, workerSource: AbstractSource)
   extends ShuffleRecoverHelper with DeviceObserver with Logging with MemoryPressureListener {
-  // mount point -> filewriter
-  val workingDirWriters = new ConcurrentHashMap[File, ConcurrentHashMap[String, FileWriter]]()
+  // mount point -> file writer
+  val workingDirWriters =
+    JavaUtils.newConcurrentHashMap[File, ConcurrentHashMap[String, FileWriter]]()
 
+  val hasHDFSStorage = conf.hasHDFSStorage
+
+  // (deviceName -> deviceInfo) and (mount point -> diskInfo)
   val (deviceInfos, diskInfos) = {
     val workingDirInfos =
       conf.workerBaseDirs.map { case (workdir, maxSpace, flusherThread, storageType) =>
         (new File(workdir, conf.workerWorkingDir), maxSpace, flusherThread, storageType)
       }
 
-    if (workingDirInfos.size <= 0) {
+    if (workingDirInfos.size <= 0 && !hasHDFSStorage) {
       throw new IOException("Empty working directory configuration!")
     }
 
@@ -78,7 +81,7 @@ final private[worker] class StorageManager(conf: CelebornConf, workerSource: Abs
     disksSnapshot().filter(_.status == DiskStatus.HEALTHY).flatMap(_.dirs)
 
   private val diskOperators: ConcurrentHashMap[String, ThreadPoolExecutor] = {
-    val cleaners = new ConcurrentHashMap[String, ThreadPoolExecutor]()
+    val cleaners = JavaUtils.newConcurrentHashMap[String, ThreadPoolExecutor]()
     disksSnapshot().foreach {
       diskInfo =>
         cleaners.put(
@@ -88,58 +91,64 @@ final private[worker] class StorageManager(conf: CelebornConf, workerSource: Abs
     cleaners
   }
 
-  val tmpDiskInfos = new ConcurrentHashMap[String, DiskInfo]()
+  val tmpDiskInfos = JavaUtils.newConcurrentHashMap[String, DiskInfo]()
   disksSnapshot().foreach { diskInfo =>
     tmpDiskInfos.put(diskInfo.mountPoint, diskInfo)
   }
   private val deviceMonitor =
-    DeviceMonitor.createDeviceMonitor(conf, this, deviceInfos, tmpDiskInfos)
+    DeviceMonitor.createDeviceMonitor(conf, this, deviceInfos, tmpDiskInfos, workerSource)
 
+  private val byteBufAllocator: PooledByteBufAllocator =
+    NettyUtils.getPooledByteBufAllocator(new TransportConf("StorageManager", conf), null, true)
   // (mountPoint -> LocalFlusher)
-  private val localFlushers: ConcurrentHashMap[String, LocalFlusher] = {
-    val flushers = new ConcurrentHashMap[String, LocalFlusher]()
+  private val (
+    localFlushers: ConcurrentHashMap[String, LocalFlusher],
+    _totalLocalFlusherThread: Int) = {
+    val flushers = JavaUtils.newConcurrentHashMap[String, LocalFlusher]()
+    var totalThread = 0;
     disksSnapshot().foreach { diskInfo =>
       if (!flushers.containsKey(diskInfo.mountPoint)) {
         val flusher = new LocalFlusher(
           workerSource,
           deviceMonitor,
           diskInfo.threadCount,
+          byteBufAllocator,
+          conf.workerPushMaxComponents,
           diskInfo.mountPoint,
           diskInfo.storageType,
           diskInfo.flushTimeMetrics)
         flushers.put(diskInfo.mountPoint, flusher)
+        totalThread = totalThread + diskInfo.threadCount
       }
     }
-    flushers
+    (flushers, totalThread)
   }
 
-  private val actionService = Executors.newSingleThreadScheduledExecutor(new ThreadFactoryBuilder()
-    .setNameFormat("StorageManager-action-thread").build)
-
   deviceMonitor.startCheck()
 
   val hdfsDir = conf.hdfsDir
-  if (!hdfsDir.isEmpty) {
-    logInfo(s"Initialize HDFS support with path ${hdfsDir}")
-  }
-  val hdfsPermission = FsPermission.createImmutable(755)
-  val hdfsWriters = new ConcurrentHashMap[String, FileWriter]()
-  val hdfsFlusher =
-    if (!hdfsDir.isEmpty) {
-      val hdfsConfiguration = new Configuration
-      hdfsConfiguration.set("fs.defaultFS", hdfsDir)
-      hdfsConfiguration.set("dfs.replication", "2")
-      StorageManager.hdfsFs = FileSystem.get(hdfsConfiguration)
-      Some(new HdfsFlusher(
-        workerSource,
-        conf.hdfsFlusherThreads))
+  val hdfsPermission = new FsPermission("755")
+  val hdfsWriters = JavaUtils.newConcurrentHashMap[String, FileWriter]()
+  val (hdfsFlusher, _totalHdfsFlusherThread) =
+    if (hasHDFSStorage) {
+      logInfo(s"Initialize HDFS support with path ${hdfsDir}")
+      StorageManager.hadoopFs = CelebornHadoopUtils.getHadoopFS(conf)
+      (
+        Some(new HdfsFlusher(
+          workerSource,
+          conf.workerHdfsFlusherThreads,
+          byteBufAllocator,
+          conf.workerPushMaxComponents)),
+        conf.workerHdfsFlusherThreads)
     } else {
-      None
+      (None, 0)
     }
 
+  def totalFlusherThread: Int = _totalLocalFlusherThread + _totalHdfsFlusherThread
+
   override def notifyError(mountPoint: String, diskStatus: DiskStatus): Unit = this.synchronized {
-    if (diskStatus == DiskStatus.IO_HANG) {
-      logInfo("IoHang, remove disk operator.")
+    if (diskStatus == DiskStatus.CRITICAL_ERROR) {
+      logInfo(s"Disk ${mountPoint} faces critical error, will remove its disk operator.")
       val operator = diskOperators.remove(mountPoint)
       if (operator != null) {
         operator.shutdown()
@@ -167,14 +176,14 @@ final private[worker] class StorageManager(conf: CelebornConf, workerSource: Abs
 
   // shuffleKey -> (fileName -> file info)
   private val fileInfos =
-    new ConcurrentHashMap[String, ConcurrentHashMap[String, FileInfo]]()
+    JavaUtils.newConcurrentHashMap[String, ConcurrentHashMap[String, FileInfo]]()
   private val RECOVERY_FILE_NAME = "recovery.ldb"
   private var db: DB = null
   // ShuffleClient can fetch data from a restarted worker only
   // when the worker's fetching port is stable.
   if (conf.workerGracefulShutdown) {
     try {
-      val recoverFile = new File(conf.workerRecoverPath, RECOVERY_FILE_NAME)
+      val recoverFile = new File(conf.workerGracefulShutdownRecoverPath, RECOVERY_FILE_NAME)
       this.db = LevelDBProvider.initLevelDB(recoverFile, CURRENT_VERSION)
       reloadAndCleanFileInfos(this.db)
     } catch {
@@ -183,19 +192,19 @@ final private[worker] class StorageManager(conf: CelebornConf, workerSource: Abs
         this.db = null
     }
   }
-  cleanupExpiredAppDirs(System.currentTimeMillis(), conf.workerGracefulShutdown)
+  cleanupExpiredAppDirs()
   if (!checkIfWorkingDirCleaned) {
     logWarning(
       "Worker still has residual files in the working directory before registering with Master, " +
-        "please refer to the configuration document to increase celeborn.worker.disk.checkFileClean.maxRetries or " +
-        "celeborn.worker.disk.checkFileClean.timeout .")
+        "please refer to the configuration document to increase " +
+        s"${CelebornConf.WORKER_CHECK_FILE_CLEAN_TIMEOUT.key}.")
   } else {
     logInfo("Successfully remove all files under working directory.")
   }
 
   private def reloadAndCleanFileInfos(db: DB): Unit = {
     if (db != null) {
-      val cache = new ConcurrentHashMap[String, UserIdentifier]()
+      val cache = JavaUtils.newConcurrentHashMap[String, UserIdentifier]()
       val itr = db.iterator
       itr.seek(SHUFFLE_KEY_PREFIX.getBytes(StandardCharsets.UTF_8))
       while (itr.hasNext) {
@@ -236,13 +245,13 @@ final private[worker] class StorageManager(conf: CelebornConf, workerSource: Abs
   private val newMapFunc =
     new java.util.function.Function[String, ConcurrentHashMap[String, FileInfo]]() {
       override def apply(key: String): ConcurrentHashMap[String, FileInfo] =
-        new ConcurrentHashMap()
+        JavaUtils.newConcurrentHashMap()
     }
 
   private val workingDirWriterListFunc =
     new java.util.function.Function[File, ConcurrentHashMap[String, FileWriter]]() {
       override def apply(t: File): ConcurrentHashMap[String, FileWriter] =
-        new ConcurrentHashMap[String, FileWriter]()
+        JavaUtils.newConcurrentHashMap[String, FileWriter]()
     }
 
   @throws[IOException]
@@ -255,7 +264,7 @@ final private[worker] class StorageManager(conf: CelebornConf, workerSource: Abs
       partitionType: PartitionType,
       rangeReadFilter: Boolean,
       userIdentifier: UserIdentifier): FileWriter = {
-    if (healthyWorkingDirs().size <= 0 && hdfsDir.isEmpty) {
+    if (healthyWorkingDirs().size <= 0 && !hasHDFSStorage) {
       throw new IOException("No available working dirs!")
     }
 
@@ -263,13 +272,13 @@ final private[worker] class StorageManager(conf: CelebornConf, workerSource: Abs
     var retryCount = 0
     var exception: IOException = null
     val suggestedMountPoint = location.getStorageInfo.getMountPoint
-    while (retryCount < conf.createWriterMaxAttempts) {
+    while (retryCount < conf.workerCreateWriterMaxAttempts) {
       val diskInfo = diskInfos.get(suggestedMountPoint)
       val dirs =
         if (diskInfo != null && diskInfo.status.equals(DiskStatus.HEALTHY)) {
           diskInfo.dirs
         } else {
-          logWarning(s"Disk unavailable for $suggestedMountPoint, return all healthy" +
+          logDebug(s"Disk unavailable for $suggestedMountPoint, return all healthy" +
             s" working dirs. diskInfo $diskInfo")
           healthyWorkingDirs()
         }
@@ -280,19 +289,31 @@ final private[worker] class StorageManager(conf: CelebornConf, workerSource: Abs
       if (dirs.isEmpty) {
         val shuffleDir =
           new Path(new Path(hdfsDir, conf.workerWorkingDir), s"$appId/$shuffleId")
-        FileSystem.mkdirs(StorageManager.hdfsFs, shuffleDir, hdfsPermission)
+        FileSystem.mkdirs(StorageManager.hadoopFs, shuffleDir, hdfsPermission)
         val fileInfo =
           new FileInfo(new Path(shuffleDir, fileName).toString, userIdentifier, partitionType)
-        val hdfsWriter = new FileWriter(
-          fileInfo,
-          hdfsFlusher.get,
-          workerSource,
-          conf,
-          deviceMonitor,
-          splitThreshold,
-          splitMode,
-          partitionType,
-          rangeReadFilter)
+        val hdfsWriter = partitionType match {
+          case PartitionType.MAP => new MapPartitionFileWriter(
+              fileInfo,
+              hdfsFlusher.get,
+              workerSource,
+              conf,
+              deviceMonitor,
+              splitThreshold,
+              splitMode,
+              rangeReadFilter)
+          case PartitionType.REDUCE => new ReducePartitionFileWriter(
+              fileInfo,
+              hdfsFlusher.get,
+              workerSource,
+              conf,
+              deviceMonitor,
+              splitThreshold,
+              splitMode,
+              rangeReadFilter)
+          case _ => throw new UnsupportedOperationException(s"Not support $partitionType yet")
+        }
+
         fileInfos.computeIfAbsent(shuffleKey, newMapFunc).put(fileName, fileInfo)
         hdfsWriters.put(fileInfo.getFilePath, hdfsWriter)
         return hdfsWriter
@@ -314,16 +335,28 @@ final private[worker] class StorageManager(conf: CelebornConf, workerSource: Abs
             }
           }
           val fileInfo = new FileInfo(file.getAbsolutePath, userIdentifier, partitionType)
-          val fileWriter = new FileWriter(
-            fileInfo,
-            localFlushers.get(mountPoint),
-            workerSource,
-            conf,
-            deviceMonitor,
-            splitThreshold,
-            splitMode,
-            partitionType,
-            rangeReadFilter)
+          fileInfo.setMountPoint(mountPoint)
+          val fileWriter = partitionType match {
+            case PartitionType.MAP => new MapPartitionFileWriter(
+                fileInfo,
+                localFlushers.get(mountPoint),
+                workerSource,
+                conf,
+                deviceMonitor,
+                splitThreshold,
+                splitMode,
+                rangeReadFilter)
+            case PartitionType.REDUCE => new ReducePartitionFileWriter(
+                fileInfo,
+                localFlushers.get(mountPoint),
+                workerSource,
+                conf,
+                deviceMonitor,
+                splitThreshold,
+                splitMode,
+                rangeReadFilter)
+            case _ => throw new UnsupportedOperationException(s"Not support $partitionType yet")
+          }
           deviceMonitor.registerFileWriter(fileWriter)
           val map = workingDirWriters.computeIfAbsent(dir, workingDirWriterListFunc)
           map.put(fileInfo.getFilePath, fileWriter)
@@ -389,6 +422,40 @@ final private[worker] class StorageManager(conf: CelebornConf, workerSource: Abs
     }.toSeq.sortBy(_._2).reverse.take(conf.metricsAppTopDiskUsageCount * 2).toMap.asJava
   }
 
+  def cleanFile(shuffleKey: String, fileName: String): Unit = {
+    val fileInfo = getFileInfo(shuffleKey, fileName)
+    if (fileInfo != null) {
+      cleanFileInternal(shuffleKey, fileInfo)
+    }
+  }
+
+  def cleanFileInternal(shuffleKey: String, fileInfo: FileInfo): Boolean = {
+    var isHdfsExpired = false
+    if (fileInfo.isHdfs) {
+      isHdfsExpired = true
+      val hdfsFileWriter = hdfsWriters.get(fileInfo.getFilePath)
+      if (hdfsFileWriter != null) {
+        hdfsFileWriter.destroy(new IOException(
+          s"Destroy FileWriter ${hdfsFileWriter} caused by shuffle ${shuffleKey} expired."))
+        hdfsWriters.remove(fileInfo.getFilePath)
+      }
+    } else {
+      val workingDir =
+        fileInfo.getFile.getParentFile.getParentFile.getParentFile
+      val writers = workingDirWriters.get(workingDir)
+      if (writers != null) {
+        val fileWriter = writers.get(fileInfo.getFilePath)
+        if (fileWriter != null) {
+          fileWriter.destroy(new IOException(
+            s"Destroy FileWriter ${fileWriter} caused by shuffle ${shuffleKey} expired."))
+          writers.remove(fileInfo.getFilePath)
+        }
+      }
+    }
+
+    isHdfsExpired
+  }
+
   def cleanupExpiredShuffleKey(expiredShuffleKeys: util.HashSet[String]): Unit = {
     expiredShuffleKeys.asScala.foreach { shuffleKey =>
       logInfo(s"Cleanup expired shuffle $shuffleKey.")
@@ -398,27 +465,8 @@ final private[worker] class StorageManager(conf: CelebornConf, workerSource: Abs
         if (removedFileInfos != null) {
           removedFileInfos.asScala.foreach {
             case (_, fileInfo) =>
-              if (fileInfo.isHdfs) {
+              if (cleanFileInternal(shuffleKey, fileInfo)) {
                 isHdfsExpired = true
-                val hdfsFileWriter = hdfsWriters.get(fileInfo.getFilePath)
-                if (hdfsFileWriter != null) {
-                  hdfsFileWriter.destroy(new IOException(
-                    s"Destroy FileWriter ${hdfsFileWriter} caused by shuffle ${shuffleKey} expired."))
-                  fileInfo.deleteAllFiles(StorageManager.hdfsFs)
-                  hdfsWriters.remove(fileInfo.getFilePath)
-                }
-              } else {
-                val workingDir =
-                  fileInfo.getFile.getParentFile.getParentFile.getParentFile
-                val writers = workingDirWriters.get(workingDir)
-                if (writers != null) {
-                  val fileWriter = writers.get(fileInfo.getFilePath)
-                  if (fileWriter != null) {
-                    fileWriter.destroy(new IOException(
-                      s"Destroy FileWriter ${fileWriter} caused by shuffle ${shuffleKey} expired."))
-                    writers.remove(fileInfo.getFilePath)
-                  }
-                }
               }
           }
         }
@@ -429,11 +477,19 @@ final private[worker] class StorageManager(conf: CelebornConf, workerSource: Abs
             deleteDirectory(file, diskOperators.get(diskInfo.mountPoint))
           }
         }
+        if (isHdfsExpired) {
+          try {
+            StorageManager.hadoopFs.delete(
+              new Path(new Path(hdfsDir, conf.workerWorkingDir), s"$appId/$shuffleId"),
+              true)
+          } catch {
+            case e: Exception => logWarning("Clean expired HDFS shuffle failed.", e)
+          }
+        }
       }
     }
   }
 
-  private val noneEmptyDirExpireDurationMs = conf.workerNonEmptyDirExpireDuration
   private val storageScheduler =
     ThreadUtils.newDaemonSingleThreadScheduledExecutor("storage-scheduler")
 
@@ -441,9 +497,8 @@ final private[worker] class StorageManager(conf: CelebornConf, workerSource: Abs
     new Runnable {
       override def run(): Unit = {
         try {
-          // Clean up dirs which has not been modified
-          // in the past {{noneEmptyExpireDurationsMs}}.
-          cleanupExpiredAppDirs(System.currentTimeMillis() - noneEmptyDirExpireDurationMs)
+          // Clean up dirs which it's application is expired.
+          cleanupExpiredAppDirs()
         } catch {
           case exception: Exception =>
             logWarning(s"Cleanup expired shuffle data exception: ${exception.getMessage}")
@@ -454,38 +509,23 @@ final private[worker] class StorageManager(conf: CelebornConf, workerSource: Abs
     30,
     TimeUnit.MINUTES)
 
-  private def cleanupExpiredAppDirs(expireTime: Long, isGracefulShutdown: Boolean = false): Unit = {
+  private def cleanupExpiredAppDirs(): Unit = {
     val appIds = shuffleKeySet().asScala.map(key => Utils.splitShuffleKey(key)._1)
     disksSnapshot().filter(_.status != DiskStatus.IO_HANG).foreach { diskInfo =>
       diskInfo.dirs.foreach {
         case workingDir if workingDir.exists() =>
           workingDir.listFiles().foreach { appDir =>
-            // Don't delete shuffleKey's data recovered from levelDB when restart with graceful shutdown
-            if (!(isGracefulShutdown && appIds.contains(appDir.getName))) {
-              if (appDir.lastModified() < expireTime) {
-                val threadPool = diskOperators.get(diskInfo.mountPoint)
-                deleteDirectory(appDir, threadPool)
-                logInfo(s"Delete expired app dir $appDir.")
-              }
+            // Don't delete shuffleKey's data that exist correct shuffle file info.
+            if (!appIds.contains(appDir.getName)) {
+              val threadPool = diskOperators.get(diskInfo.mountPoint)
+              deleteDirectory(appDir, threadPool)
+              logInfo(s"Delete expired app dir $appDir.")
             }
           }
         // workingDir not exist when initializing worker on new disk
         case _ => // do nothing
       }
     }
-
-    if (hdfsFs != null) {
-      val hdfsWorkPath = new Path(hdfsDir, conf.workerWorkingDir)
-      if (hdfsFs.exists(hdfsWorkPath)) {
-        val iter = hdfsFs.listFiles(hdfsWorkPath, false)
-        while (iter.hasNext) {
-          val fileStatus = iter.next()
-          if (fileStatus.getModificationTime < expireTime) {
-            hdfsFs.delete(fileStatus.getPath, true)
-          }
-        }
-      }
-    }
   }
 
   private def deleteDirectory(dir: File, threadPool: ThreadPoolExecutor): Unit = {
@@ -537,10 +577,10 @@ final private[worker] class StorageManager(conf: CelebornConf, workerSource: Abs
           }
         }
 
-      val hdfsCleaned = hdfsFs match {
+      val hdfsCleaned = hadoopFs match {
         case hdfs: FileSystem =>
           val hdfsWorkPath = new Path(hdfsDir, conf.workerWorkingDir)
-          // hdfs path not exist when first time initialize
+          // HDFS path not exist when first time initialize
           if (hdfs.exists(hdfsWorkPath)) {
             !hdfs.listFiles(hdfsWorkPath, false).hasNext
           } else {
@@ -583,7 +623,7 @@ final private[worker] class StorageManager(conf: CelebornConf, workerSource: Abs
         diskOperators.size()) { entry =>
         ThreadUtils.shutdown(
           entry._2,
-          conf.workerFlusherShutdownTimeoutMs.milliseconds)
+          conf.workerGracefulShutdownFlusherShutdownTimeoutMs.milliseconds)
       }
     }
     storageScheduler.shutdownNow()
@@ -593,24 +633,32 @@ final private[worker] class StorageManager(conf: CelebornConf, workerSource: Abs
   }
 
   private def flushFileWriters(): Unit = {
-    val allWriters = new util.HashSet[FileWriter]()
-    workingDirWriters.asScala.foreach { case (_, writers) =>
-      writers.synchronized {
-        // Filter out FileWriter that already has IOException to avoid printing too many error logs
-        allWriters.addAll(writers.values().asScala.filter(_.getException == null).asJavaCollection)
+    workingDirWriters.forEach(new BiConsumer[File, ConcurrentHashMap[String, FileWriter]] {
+      override def accept(t: File, writers: ConcurrentHashMap[String, FileWriter]): Unit = {
+        writers.forEach(new BiConsumer[String, FileWriter] {
+          override def accept(file: String, writer: FileWriter): Unit = {
+            if (writer.getException == null) {
+              try {
+                writer.flushOnMemoryPressure()
+              } catch {
+                case t: Throwable =>
+                  logError(
+                    s"FileWrite of $writer faces unexpected exception when flush on memory pressure.",
+                    t)
+              }
+            } else {
+              logWarning(s"Skip flushOnMemoryPressure because ${writer.flusher} " +
+                s"has error: ${writer.getException.getMessage}")
+            }
+          }
+        })
       }
-    }
-
-    allWriters.asScala.foreach { writer =>
-      try {
-        writer.flushOnMemoryPressure()
-      } catch {
-        case t: Throwable =>
-          logError(
-            s"FileWrite of ${writer} faces unexpected exception when flush on memory pressure.",
-            t)
+    })
+    hdfsWriters.forEach(new BiConsumer[String, FileWriter] {
+      override def accept(t: String, u: FileWriter): Unit = {
+        u.flushOnMemoryPressure();
       }
-    }
+    })
   }
 
   override def onPause(moduleName: String): Unit = {}
@@ -618,11 +666,13 @@ final private[worker] class StorageManager(conf: CelebornConf, workerSource: Abs
   override def onResume(moduleName: String): Unit = {}
 
   override def onTrim(): Unit = {
-    actionService.submit(new Runnable {
-      override def run(): Unit = {
-        flushFileWriters()
-      }
-    })
+    logInfo(s"Trigger ${this.getClass.getCanonicalName} trim action")
+    flushFileWriters()
+    try {
+      Thread.sleep(conf.workerDirectMemoryTrimFlushWaitInterval)
+    } catch {
+      case _: Exception => // Do nothing
+    }
   }
 
   def updateDiskInfos(): Unit = this.synchronized {
@@ -646,7 +696,7 @@ final private[worker] class StorageManager(conf: CelebornConf, workerSource: Abs
       diskInfo.updateFlushTime()
       diskInfo.updateFetchTime()
     }
-    logInfo(s"Updated diskInfos: ${disksSnapshot()}")
+    logInfo(s"Updated diskInfos:\n${disksSnapshot().mkString("\n")}")
   }
 
   def userResourceConsumptionSnapshot(): Map[UserIdentifier, ResourceConsumption] = {
@@ -683,5 +733,5 @@ final private[worker] class StorageManager(conf: CelebornConf, workerSource: Abs
 }
 
 object StorageManager {
-  var hdfsFs: FileSystem = _
+  var hadoopFs: FileSystem = _
 }
