@@ -57,8 +57,6 @@ import org.slf4j.LoggerFactory;
 import org.apache.celeborn.common.CelebornConf;
 import org.apache.celeborn.common.identity.UserIdentifier;
 import org.apache.celeborn.common.meta.FileInfo;
-import org.apache.celeborn.common.metrics.MetricsSystem;
-import org.apache.celeborn.common.metrics.source.RPCSource;
 import org.apache.celeborn.common.network.TransportContext;
 import org.apache.celeborn.common.network.buffer.ManagedBuffer;
 import org.apache.celeborn.common.network.client.ChunkReceivedCallback;
@@ -68,16 +66,17 @@ import org.apache.celeborn.common.network.protocol.Message;
 import org.apache.celeborn.common.network.protocol.OpenStream;
 import org.apache.celeborn.common.network.protocol.StreamHandle;
 import org.apache.celeborn.common.network.server.TransportServer;
-import org.apache.celeborn.common.network.server.memory.MemoryManager;
-import org.apache.celeborn.common.network.util.JavaUtils;
+import org.apache.celeborn.common.network.util.NettyUtils;
 import org.apache.celeborn.common.network.util.TransportConf;
 import org.apache.celeborn.common.protocol.PartitionSplitMode;
 import org.apache.celeborn.common.protocol.PartitionType;
 import org.apache.celeborn.common.protocol.StorageInfo;
+import org.apache.celeborn.common.util.JavaUtils;
 import org.apache.celeborn.common.util.ThreadUtils;
 import org.apache.celeborn.common.util.Utils;
 import org.apache.celeborn.service.deploy.worker.FetchHandler;
 import org.apache.celeborn.service.deploy.worker.WorkerSource;
+import org.apache.celeborn.service.deploy.worker.memory.MemoryManager;
 
 public class FileWriterSuiteJ {
 
@@ -103,7 +102,7 @@ public class FileWriterSuiteJ {
   @BeforeClass
   public static void beforeAll() {
     tempDir = Utils.createTempDir(System.getProperty("java.io.tmpdir"), "celeborn");
-    CONF.set("celeborn.shuffle.chuck.size", "1k");
+    CONF.set(CelebornConf.SHUFFLE_CHUNK_SIZE().key(), "1k");
 
     source = Mockito.mock(WorkerSource.class);
     Mockito.doAnswer(
@@ -118,13 +117,31 @@ public class FileWriterSuiteJ {
     dirs.$plus$eq(tempDir);
     localFlusher =
         new LocalFlusher(
-            source, DeviceMonitor$.MODULE$.EmptyMonitor(), 1, "disk1", StorageInfo.Type.HDD, null);
-    MemoryManager.initialize(0.8, 0.9, 0.5, 0.6, 0.1, 0.1, 10, 10);
+            source,
+            DeviceMonitor$.MODULE$.EmptyMonitor(),
+            1,
+            NettyUtils.getPooledByteBufAllocator(new TransportConf("test", CONF), null, true),
+            256,
+            "disk1",
+            StorageInfo.Type.HDD,
+            null);
+
+    CelebornConf conf = new CelebornConf();
+    conf.set(CelebornConf.WORKER_DIRECT_MEMORY_RATIO_PAUSE_RECEIVE().key(), "0.8");
+    conf.set(CelebornConf.WORKER_DIRECT_MEMORY_RATIO_PAUSE_REPLICATE().key(), "0.9");
+    conf.set(CelebornConf.WORKER_DIRECT_MEMORY_RATIO_RESUME().key(), "0.5");
+    conf.set(CelebornConf.PARTITION_SORTER_DIRECT_MEMORY_RATIO_THRESHOLD().key(), "0.6");
+    conf.set(CelebornConf.WORKER_DIRECT_MEMORY_RATIO_FOR_READ_BUFFER().key(), "0.1");
+    conf.set(CelebornConf.WORKER_DIRECT_MEMORY_RATIO_FOR_SHUFFLE_STORAGE().key(), "0.1");
+    conf.set(CelebornConf.WORKER_DIRECT_MEMORY_CHECK_INTERVAL().key(), "10");
+    conf.set(CelebornConf.WORKER_DIRECT_MEMORY_REPORT_INTERVAL().key(), "10");
+    conf.set(CelebornConf.WORKER_READBUFFER_ALLOCATIONWAIT().key(), "10ms");
+    MemoryManager.initialize(conf);
   }
 
-  public static void setupChunkServer(FileInfo info) throws Exception {
+  public static void setupChunkServer(FileInfo info) {
     FetchHandler handler =
-        new FetchHandler(transConf) {
+        new FetchHandler(transConf.getCelebornConf(), transConf) {
           @Override
           public StorageManager storageManager() {
             return new StorageManager(CONF, source);
@@ -140,11 +157,6 @@ public class FileWriterSuiteJ {
             return source;
           }
 
-          @Override
-          public RPCSource rpcSource() {
-            return new RPCSource(CONF, MetricsSystem.ROLE_WORKER());
-          }
-
           @Override
           public boolean checkRegistered() {
             return true;
@@ -206,9 +218,9 @@ public class FileWriterSuiteJ {
     final Semaphore sem = new Semaphore(0);
 
     final FetchResult res = new FetchResult();
-    res.successChunks = Collections.synchronizedSet(new HashSet<Integer>());
-    res.failedChunks = Collections.synchronizedSet(new HashSet<Integer>());
-    res.buffers = Collections.synchronizedList(new LinkedList<ManagedBuffer>());
+    res.successChunks = Collections.synchronizedSet(new HashSet<>());
+    res.failedChunks = Collections.synchronizedSet(new HashSet<>());
+    res.buffers = Collections.synchronizedList(new LinkedList<>());
 
     ChunkReceivedCallback callback =
         new ChunkReceivedCallback() {
@@ -228,7 +240,7 @@ public class FileWriterSuiteJ {
         };
 
     for (int chunkIndex : chunkIndices) {
-      client.fetchChunk(streamId, chunkIndex, callback);
+      client.fetchChunk(streamId, chunkIndex, 10000, callback);
     }
     if (!sem.tryAcquire(chunkIndices.size(), 5, TimeUnit.SECONDS)) {
       fail("Timeout getting response from the server");
@@ -243,7 +255,7 @@ public class FileWriterSuiteJ {
     final int threadsNum = 8;
     File file = getTemporaryFile();
     FileWriter fileWriter =
-        new FileWriter(
+        new ReducePartitionFileWriter(
             new FileInfo(file, userIdentifier),
             localFlusher,
             source,
@@ -251,7 +263,6 @@ public class FileWriterSuiteJ {
             DeviceMonitor$.MODULE$.EmptyMonitor(),
             SPLIT_THRESHOLD,
             splitMode,
-            partitionType,
             false);
 
     List<Future<?>> futures = new ArrayList<>();
@@ -283,12 +294,57 @@ public class FileWriterSuiteJ {
   }
 
   @Test
-  public void testAfterStressfulWriteWillReadCorrect()
+  public void testMultiThreadWriteDuringClose()
       throws IOException, ExecutionException, InterruptedException {
-    final int threadsNum = Runtime.getRuntime().availableProcessors();
+    final int threadsNum = 8;
     File file = getTemporaryFile();
     FileWriter fileWriter =
-        new FileWriter(
+        new ReducePartitionFileWriter(
+            new FileInfo(file, userIdentifier),
+            localFlusher,
+            source,
+            CONF,
+            DeviceMonitor$.MODULE$.EmptyMonitor(),
+            SPLIT_THRESHOLD,
+            splitMode,
+            false);
+
+    List<Future<?>> futures = new ArrayList<>();
+    ExecutorService es = ThreadUtils.newDaemonFixedThreadPool(threadsNum, "FileWriter-UT-1");
+    AtomicLong length = new AtomicLong(0);
+    AtomicLong bytesWritten = new AtomicLong(0);
+
+    for (int i = 0; i < threadsNum; ++i) {
+      futures.add(
+          es.submit(
+              () -> {
+                byte[] bytes = generateData();
+                ByteBuf buf = Unpooled.wrappedBuffer(bytes);
+                try {
+                  fileWriter.write(buf);
+                  length.addAndGet(bytes.length);
+                  bytesWritten.set(fileWriter.close());
+                } catch (IOException e) {
+                  LOG.error("Failed to write buffer.", e);
+                }
+              }));
+    }
+    for (Future<?> future : futures) {
+      future.get();
+    }
+
+    assertEquals(length.get(), bytesWritten.get());
+    assertEquals(fileWriter.getFile().length(), bytesWritten.get());
+    assertEquals(fileWriter.getFileInfo().getFileLength(), bytesWritten.get());
+  }
+
+  @Test
+  public void testAfterStressfulWriteWillReadCorrect()
+      throws IOException, ExecutionException, InterruptedException {
+    final int threadsNum = Runtime.getRuntime().availableProcessors();
+    File file = getTemporaryFile();
+    FileWriter fileWriter =
+        new ReducePartitionFileWriter(
             new FileInfo(file, userIdentifier),
             localFlusher,
             source,
@@ -296,7 +352,6 @@ public class FileWriterSuiteJ {
             DeviceMonitor$.MODULE$.EmptyMonitor(),
             SPLIT_THRESHOLD,
             splitMode,
-            partitionType,
             false);
 
     List<Future<?>> futures = new ArrayList<>();
@@ -333,7 +388,14 @@ public class FileWriterSuiteJ {
     dirs.$plus$eq(file);
     localFlusher =
         new LocalFlusher(
-            source, DeviceMonitor$.MODULE$.EmptyMonitor(), 1, "disk2", StorageInfo.Type.HDD, null);
+            source,
+            DeviceMonitor$.MODULE$.EmptyMonitor(),
+            1,
+            NettyUtils.getPooledByteBufAllocator(new TransportConf("test", CONF), null, true),
+            256,
+            "disk2",
+            StorageInfo.Type.HDD,
+            null);
   }
 
   @Test
@@ -342,7 +404,7 @@ public class FileWriterSuiteJ {
     File file = getTemporaryFile();
     FileInfo fileInfo = new FileInfo(file, userIdentifier);
     FileWriter fileWriter =
-        new FileWriter(
+        new ReducePartitionFileWriter(
             fileInfo,
             localFlusher,
             source,
@@ -350,7 +412,6 @@ public class FileWriterSuiteJ {
             DeviceMonitor$.MODULE$.EmptyMonitor(),
             SPLIT_THRESHOLD,
             splitMode,
-            partitionType,
             false);
 
     List<Future<?>> futures = new ArrayList<>();
@@ -442,6 +503,212 @@ public class FileWriterSuiteJ {
     assertEquals(0, compositeByteBuf.numComponents());
   }
 
+  @Test
+  public void testChunkSize() throws IOException {
+    // NOTE: SHUFFLE_CHUNK_SIZE and WORKER_FLUSHER_BUFFER_SIZE are set to 1K/128B
+    CelebornConf conf = new CelebornConf();
+    conf.set(CelebornConf.SHUFFLE_CHUNK_SIZE().key(), "1k");
+    conf.set(CelebornConf.WORKER_FLUSHER_BUFFER_SIZE().key(), "128B");
+
+    // case 1: write 8MiB
+    File file = getTemporaryFile();
+    FileInfo fileInfo = new FileInfo(file, userIdentifier);
+    FileWriter fileWriter =
+        new ReducePartitionFileWriter(
+            fileInfo,
+            localFlusher,
+            source,
+            conf,
+            DeviceMonitor$.MODULE$.EmptyMonitor(),
+            SPLIT_THRESHOLD,
+            splitMode,
+            false);
+    fileWriter.write(generateData(8 * 1024 * 1024));
+    fileWriter.close();
+    assertEquals(fileInfo.numChunks(), 1);
+    assertEquals(fileInfo.getLastChunkOffset(), 8 * 1024 * 1024);
+    assertEquals(
+        fileInfo.getChunkOffsets().get(1) - fileInfo.getChunkOffsets().get(0), 8 * 1024 * 1024);
+
+    // case 2: write 1024B
+    file = getTemporaryFile();
+    fileInfo = new FileInfo(file, userIdentifier);
+    fileWriter =
+        new ReducePartitionFileWriter(
+            fileInfo,
+            localFlusher,
+            source,
+            conf,
+            DeviceMonitor$.MODULE$.EmptyMonitor(),
+            SPLIT_THRESHOLD,
+            splitMode,
+            false);
+    for (int i = 0; i < 8; i++) {
+      fileWriter.write(generateData(128));
+    }
+    fileWriter.close();
+    assertEquals(fileInfo.numChunks(), 1);
+    assertEquals(fileInfo.getLastChunkOffset(), 1024);
+    assertEquals(fileInfo.getChunkOffsets().get(1) - fileInfo.getChunkOffsets().get(0), 1024);
+
+    // case 3: write 1023B
+    file = getTemporaryFile();
+    fileInfo = new FileInfo(file, userIdentifier);
+    fileWriter =
+        new ReducePartitionFileWriter(
+            fileInfo,
+            localFlusher,
+            source,
+            conf,
+            DeviceMonitor$.MODULE$.EmptyMonitor(),
+            SPLIT_THRESHOLD,
+            splitMode,
+            false);
+    fileWriter.write(generateData(1020));
+    fileWriter.write(generateData(3));
+    fileWriter.close();
+    assertEquals(fileInfo.numChunks(), 1);
+    assertEquals(fileInfo.getLastChunkOffset(), 1023);
+    assertEquals(fileInfo.getChunkOffsets().get(1) - fileInfo.getChunkOffsets().get(0), 1023);
+
+    // case 4: write 1025B
+    file = getTemporaryFile();
+    fileInfo = new FileInfo(file, userIdentifier);
+    fileWriter =
+        new ReducePartitionFileWriter(
+            fileInfo,
+            localFlusher,
+            source,
+            conf,
+            DeviceMonitor$.MODULE$.EmptyMonitor(),
+            SPLIT_THRESHOLD,
+            splitMode,
+            false);
+    for (int i = 0; i < 8; i++) {
+      fileWriter.write(generateData(128));
+    }
+    fileWriter.write(generateData(1));
+    fileWriter.close();
+    assertEquals(fileInfo.numChunks(), 2);
+    assertEquals(fileInfo.getLastChunkOffset(), 1025);
+    assertEquals(fileInfo.getChunkOffsets().get(1) - fileInfo.getChunkOffsets().get(0), 1024);
+
+    // case 5: write 2048B
+    file = getTemporaryFile();
+    fileInfo = new FileInfo(file, userIdentifier);
+    fileWriter =
+        new ReducePartitionFileWriter(
+            fileInfo,
+            localFlusher,
+            source,
+            conf,
+            DeviceMonitor$.MODULE$.EmptyMonitor(),
+            SPLIT_THRESHOLD,
+            splitMode,
+            false);
+    for (int i = 0; i < 16; i++) {
+      fileWriter.write(generateData(128));
+    }
+    fileWriter.close();
+    assertEquals(fileInfo.numChunks(), 2);
+    assertEquals(fileInfo.getLastChunkOffset(), 2048);
+    assertEquals(fileInfo.getChunkOffsets().get(1) - fileInfo.getChunkOffsets().get(0), 1024);
+
+    // case 5.1: write 2048B with trim; without PR #1702 this case will fail
+    file = getTemporaryFile();
+    fileInfo = new FileInfo(file, userIdentifier);
+    fileWriter =
+        new ReducePartitionFileWriter(
+            fileInfo,
+            localFlusher,
+            source,
+            conf,
+            DeviceMonitor$.MODULE$.EmptyMonitor(),
+            SPLIT_THRESHOLD,
+            splitMode,
+            false);
+    for (int i = 0; i < 16; i++) {
+      fileWriter.write(generateData(128));
+    }
+    // mock trim
+    fileWriter.flush(false);
+    fileWriter.close();
+    assertEquals(fileInfo.numChunks(), 2);
+    assertEquals(fileInfo.getLastChunkOffset(), 2048);
+    assertEquals(fileInfo.getChunkOffsets().get(1) - fileInfo.getChunkOffsets().get(0), 1024);
+
+    // case 6: write 2049B
+    file = getTemporaryFile();
+    fileInfo = new FileInfo(file, userIdentifier);
+    fileWriter =
+        new ReducePartitionFileWriter(
+            fileInfo,
+            localFlusher,
+            source,
+            conf,
+            DeviceMonitor$.MODULE$.EmptyMonitor(),
+            SPLIT_THRESHOLD,
+            splitMode,
+            false);
+    for (int i = 0; i < 16; i++) {
+      fileWriter.write(generateData(128));
+    }
+    fileWriter.write(generateData(1));
+    fileWriter.close();
+    assertEquals(fileInfo.numChunks(), 3);
+    assertEquals(fileInfo.getLastChunkOffset(), 2049);
+    assertEquals(fileInfo.getChunkOffsets().get(2) - fileInfo.getChunkOffsets().get(1), 1024);
+
+    // case 7: write 4097B with 3 chunks
+    file = getTemporaryFile();
+    fileInfo = new FileInfo(file, userIdentifier);
+    fileWriter =
+        new ReducePartitionFileWriter(
+            fileInfo,
+            localFlusher,
+            source,
+            conf,
+            DeviceMonitor$.MODULE$.EmptyMonitor(),
+            SPLIT_THRESHOLD,
+            splitMode,
+            false);
+    fileWriter.write(generateData(1024));
+    for (int i = 0; i < 9; i++) {
+      fileWriter.write(generateData(128));
+    }
+    fileWriter.write(generateData(1920));
+    fileWriter.close();
+    assertEquals(fileInfo.numChunks(), 3);
+    assertEquals(fileInfo.getLastChunkOffset(), 4096);
+    assertEquals(fileInfo.getChunkOffsets().get(3) - fileInfo.getChunkOffsets().get(2), 2048);
+
+    // case 7.2: write 4097B with 3 chunks with trim; without PR #1702 this case will fail
+    file = getTemporaryFile();
+    fileInfo = new FileInfo(file, userIdentifier);
+    fileWriter =
+        new ReducePartitionFileWriter(
+            fileInfo,
+            localFlusher,
+            source,
+            conf,
+            DeviceMonitor$.MODULE$.EmptyMonitor(),
+            SPLIT_THRESHOLD,
+            splitMode,
+            false);
+    fileWriter.write(generateData(1024));
+    for (int i = 0; i < 9; i++) {
+      fileWriter.write(generateData(128));
+      fileWriter.flush(false);
+    }
+    fileWriter.write(generateData(1920));
+    // mock trim
+    fileWriter.flush(false);
+    fileWriter.close();
+    assertEquals(fileInfo.numChunks(), 3);
+    assertEquals(fileInfo.getLastChunkOffset(), 4096);
+    assertEquals(fileInfo.getChunkOffsets().get(3) - fileInfo.getChunkOffsets().get(2), 2048);
+  }
+
   private File getTemporaryFile() throws IOException {
     String filename = UUID.randomUUID().toString();
     File temporaryFile = new File(tempDir, filename);
@@ -461,4 +728,12 @@ public class FileWriterSuiteJ {
     }
     return data;
   }
+
+  private ByteBuf generateData(int len) {
+    byte[] data = new byte[len];
+    for (int i = 0; i < len; i++) {
+      data[i] = 'a';
+    }
+    return Unpooled.wrappedBuffer(data);
+  }
 }
